@@ -2,7 +2,7 @@
  * audio_file_device.h - base-class for audio-device-classes which write 
  *                       their output into a file
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,19 +27,8 @@
 #define _AUDIO_FILE_DEVICE_H
 
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QFile>
 
-#else
-
-#include <qfile.h>
-
-#endif
-
-
 #include "audio_device.h"
 
 
@@ -53,14 +42,18 @@ public:
 				const bitrate_t _nom_bitrate,
 				const bitrate_t _min_bitrate,
 				const bitrate_t _max_bitrate,
+				const int _depth,
 				mixer * _mixer );
 	virtual ~audioFileDevice();
 
+	QString outputFile( void ) const
+	{
+		return( m_outputFile.fileName() );
+	}
 
 
 protected:
-	Sint32 FASTCALL writeData( const void * _data, Sint32 _len );
-	void seekToBegin( void );
+	Sint32 writeData( const void * _data, Sint32 _len );
 
 	inline bool useVBR( void ) const
 	{
@@ -82,6 +75,11 @@ protected:
 		return( m_maxBitrate );
 	}
 
+	inline int depth( void ) const
+	{
+		return( m_depth );
+	}
+
 	inline bool outputFileOpened( void ) const
 	{
 		return( m_outputFile.isOpen() );
@@ -97,7 +95,22 @@ private:
 	bitrate_t m_minBitrate;
 	bitrate_t m_maxBitrate;
 
+	int m_depth;
+
 } ;
 
 
+typedef audioFileDevice * ( * audioFileDeviceInstantiaton )
+					( const sample_rate_t _sample_rate,
+						const ch_cnt_t _channels,
+						bool & _success_ful,
+						const QString & _file,
+						const bool _use_vbr,
+						const bitrate_t _nom_bitrate,
+						const bitrate_t _min_bitrate,
+						const bitrate_t _max_bitrate,
+						const int _depth,
+						mixer * _mixer );
+
+
 #endif
