@@ -2,7 +2,7 @@
  * audio_file_wave.h - Audio-device which encodes wave-stream and writes it
  *                     into an WAVE-file. This is used for song-export.
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,8 +28,10 @@
 #define _AUDIO_FILE_WAVE_H
 
 
+#include "lmmsconfig.h"
 #include "audio_file_device.h"
 
+#include <sndfile.h>
 
 
 class audioFileWave : public audioFileDevice
@@ -43,6 +45,7 @@ public:
 			const bitrate_t _nom_bitrate,
 			const bitrate_t _min_bitrate,
 			const bitrate_t _max_bitrate,
+			const int _depth,
 			mixer * _mixer );
 	virtual ~audioFileWave();
 
@@ -54,18 +57,19 @@ public:
 						const bitrate_t _nom_bitrate,
 						const bitrate_t _min_bitrate,
 						const bitrate_t _max_bitrate,
+						const int _depth,
 						mixer * _mixer )
 	{
 		return( new audioFileWave( _sample_rate, _channels,
 						_success_ful, _file, _use_vbr,
 						_nom_bitrate, _min_bitrate,
-							_max_bitrate,
+							_max_bitrate, _depth,
 							_mixer ) );
 	}
 
 
 private:
-	virtual void FASTCALL writeBuffer( const surroundSampleFrame * _ab,
+	virtual void writeBuffer( const surroundSampleFrame * _ab,
 						const fpp_t _frames,
 						float _master_gain );
 
@@ -73,24 +77,8 @@ private:
 	void finishEncoding( void );
 
 
-	int m_bytesWritten;
-
-	struct waveFileHeader
-	{
-		char riff_id[4];		// "RIFF"
-		Uint32 total_bytes;		// total filesize-8
-		char wave_fmt_str[8];		// "WAVEfmt"
-		Uint32 bitrate_1;		// bitrate, e.g. 16
-		Uint16 uncompressed;		// 1 if PCM
-		Uint16 channels;		// 1 = mono  2 = stereo
-		Uint32 sample_rate;		// sample-rate e.g. 44100
-		Uint32 bytes_per_second;	// sample-rate*channels*
-						// (bitrate/8)
-		Uint16 block_alignment;		// channels*(bitrate/8)
-		Uint16 bitrate_2;		// bitrate, e.g. 16
-		char data_chunk_id[4];		// "data"
-		Uint32 data_bytes;		// total size of sample-data
-	} m_waveFileHeader;
+	SF_INFO m_si;
+	SNDFILE * m_sf;
 
 } ;
 
