@@ -1,7 +1,7 @@
 /*
  * audio_sdl.h - device-class that performs PCM-output via SDL
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,20 +26,12 @@
 #ifndef _AUDIO_SDL_H
 #define _AUDIO_SDL_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "lmmsconfig.h"
 
-#ifdef HAVE_SDL_SDL_AUDIO_H
-
-#ifndef SDL_AUDIO_SUPPORT
-#define SDL_AUDIO_SUPPORT
-#endif
-
-
-#include SDL_SDL_H
-#include SDL_SDL_AUDIO_H
+#ifdef LMMS_HAVE_SDL
 
+#include <SDL/SDL.h>
+#include <SDL/SDL_audio.h>
 
 #include "audio_device.h"
 
@@ -50,14 +42,13 @@ class QLineEdit;
 class audioSDL : public audioDevice
 {
 public:
-	audioSDL( const sample_rate_t _sample_rate, bool & _success_ful,
-							mixer * _mixer );
+	audioSDL( bool & _success_ful, mixer * _mixer );
 	virtual ~audioSDL();
 
 	inline static QString name( void )
 	{
 		return( QT_TRANSLATE_NOOP( "setupWidget",
-			"SDL (Simple DirectMedia Layer)" ) );
+					"SDL (Simple DirectMedia Layer)" ) );
 	}
 
 
@@ -78,6 +69,7 @@ public:
 private:
 	virtual void startProcessing( void );
 	virtual void stopProcessing( void );
+	virtual void applyQualitySettings( void );
 
 	static void sdlAudioCallback( void * _udata, Uint8 * _buf, int _len );
 	void sdlAudioCallback( Uint8 * _buf, int _len );
@@ -86,13 +78,13 @@ private:
 
 	surroundSampleFrame * m_outBuf;
 	Uint8 * m_convertedBuf;
-	int m_convertedBuf_pos;
-	int m_convertedBuf_size;
+	int m_convertedBufPos;
+	int m_convertedBufSize;
 
 	bool m_convertEndian;
 
-	bool m_stopped;
-	QSemaphore m_stop_semaphore;
+	volatile bool m_stopped;
+	QSemaphore m_stopSemaphore;
 
 } ;
 
