@@ -1,7 +1,7 @@
 /*
  * automatable_button.h - class automatableButton, the base for all buttons
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,59 +26,38 @@
 #ifndef _AUTOMATABLE_BUTTON_H
 #define _AUTOMATABLE_BUTTON_H
 
-#include "qt3support.h"
+#include <QtGui/QPushButton>
 
-#ifdef QT4
-
-#include <QtGui/QWidget>
-
-#else
-
-#include <qwidget.h>
-
-#endif
-
-
-#include "automatable_object.h"
+#include "automatable_model_view.h"
 
 
 class automatableButtonGroup;
 
 
-class automatableButton : public QWidget, public automatableObject<bool,
-			  					signed char>
+class EXPORT automatableButton : public QPushButton, public boolModelView
 {
 	Q_OBJECT
 public:
-	automatableButton( QWidget * _parent, const QString & _name,
-							track * _track );
+	automatableButton( QWidget * _parent, const QString & _name 
+			= QString::null );
 	virtual ~automatableButton();
 
-
-	inline virtual bool isChecked( void ) const
-	{
-		return( value() );
-	}
-
-	virtual void setValue( const bool _on );
-
 	inline void setCheckable( bool _on )
 	{
-		m_checkable = _on;
-		setJournalling( m_checkable );
+		QPushButton::setCheckable( _on );
+		model()->setJournalling( _on );
 	}
 
-	inline bool isCheckable( void ) const
-	{
-		return( m_checkable );
-	}
+	virtual void modelChanged( void );
 
 
 public slots:
+	virtual void update( void );
 	virtual void toggle( void );
 	virtual void setChecked( bool _on )
 	{
-		setValue( _on );
+		// QPushButton::setChecked is called in update-slot
+		model()->setValue( _on );
 	}
 
 
@@ -90,26 +69,22 @@ protected:
 
 private:
 	automatableButtonGroup * m_group;
-	bool m_checkable;
 
 
 	friend class automatableButtonGroup;
 
-
-signals:
-	void clicked( void );
-	void toggled( bool );
-
+	using QPushButton::setChecked;
+	using QPushButton::isChecked;
 } ;
 
 
 
-class automatableButtonGroup : public QWidget, public automatableObject<int>
+class EXPORT automatableButtonGroup : public QWidget, public intModelView
 {
 	Q_OBJECT
 public:
-	automatableButtonGroup( QWidget * _parent, const QString & _name,
-							track * _track );
+	automatableButtonGroup( QWidget * _parent, const QString & _name
+			= QString::null );
 	virtual ~automatableButtonGroup();
 
 	void addButton( automatableButton * _btn );
@@ -117,15 +92,15 @@ public:
 
 	void activateButton( automatableButton * _btn );
 
-	virtual void setValue( const int _value );
+	virtual void modelChanged( void );
+
+
+private slots:
+	void updateButtons( void );
 
 
 private:
-	vlist<automatableButton *> m_buttons;
-
-
-signals:
-	void valueChanged( int );
+	QList<automatableButton *> m_buttons;
 
 } ;
 
