@@ -2,7 +2,7 @@
  * automation_editor.h - declaration of class automationEditor which is a window
  *                       where you can edit dynamic values in an easy way
  *
- * Copyright (c) 2006-2007 Javier Serrano Polo <jasp00/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Javier Serrano Polo <jasp00/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,24 +27,13 @@
 #ifndef _AUTOMATION_EDITOR_H
 #define _AUTOMATION_EDITOR_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QWidget>
-#include <QtCore/QVector>
 
-#else
-
-#include <qwidget.h>
-#include <qvaluevector.h>
-
-#endif
-
-#include "types.h"
+#include "lmms_basics.h"
 #include "journalling_object.h"
 #include "midi_time.h"
 #include "automation_pattern.h"
+#include "combobox_model.h"
 
 
 class QPainter;
@@ -61,7 +50,7 @@ class automationEditor : public QWidget, public journallingObject
 {
 	Q_OBJECT
 public:
-	void FASTCALL setCurrentPattern( automationPattern * _new_pattern );
+	void setCurrentPattern( automationPattern * _new_pattern );
 
 	inline const automationPattern * currentPattern( void ) const
 	{
@@ -76,12 +65,11 @@ public:
 	int quantization( void ) const;
 
 
-	virtual void FASTCALL saveSettings( QDomDocument & _doc,
-							QDomElement & _parent );
-	virtual void FASTCALL loadSettings( const QDomElement & _this );
+	virtual void saveSettings( QDomDocument & _doc, QDomElement & _parent );
+	virtual void loadSettings( const QDomElement & _this );
 	inline virtual QString nodeName( void ) const
 	{
-		return( "automation-editor" );
+		return( "automationeditor" );
 	}
 
 
@@ -102,14 +90,15 @@ protected:
 	virtual void resizeEvent( QResizeEvent * _re );
 	virtual void wheelEvent( QWheelEvent * _we );
 
-	int FASTCALL getLevel( int _y );
-	static inline void drawValueRect( QPainter & _p, Uint16 _x, Uint16 _y,
-						Sint16 _width, Sint16 _height,
+	float getLevel( int _y );
+	static inline void drawValueRect( QPainter & _p, int _x, int _y,
+						int _width, int _height,
 						const bool _is_selected );
 	void removeSelection( void );
 	void selectAll( void );
-	void FASTCALL getSelectedValues( timeMap & _selected_values );
+	void getSelectedValues( timeMap & _selected_values );
 
+	void drawLine( int x0, float y0, int x1, float y1 );
 
 protected slots:
 	void play( void );
@@ -130,8 +119,8 @@ protected slots:
 
 	void updatePosition( const midiTime & _t );
 
-	void zoomingXChanged( const QString & _zfac );
-	void zoomingYChanged( const QString & _zfac );
+	void zoomingXChanged( void );
+	void zoomingYChanged( void );
 
 
 private:
@@ -153,7 +142,7 @@ private:
 	} ;
 
 	// some constants...
-	static const int INITIAL_WIDTH = 640;
+	static const int INITIAL_WIDTH = 740;
 	static const int INITIAL_HEIGHT = 480;
 
 	static const int SCROLLBAR_SIZE = 16;
@@ -194,15 +183,18 @@ private:
 	comboBox * m_zoomingYComboBox;
 	comboBox * m_quantizeComboBox;
 
+	comboBoxModel m_zoomingXModel;
+	comboBoxModel m_zoomingYModel;
+	comboBoxModel m_quantizeModel;
 
 	automationPattern * m_pattern;
-	int m_min_level;
-	int m_max_level;
-	int m_scroll_level;
-	int m_bottom_level;
-	int m_top_level;
+	float m_minLevel;
+	float m_maxLevel;
+	float m_step;
+	float m_scrollLevel;
+	float m_bottomLevel;
+	float m_topLevel;
 
-	void updatePaintPixmap( QPixmap & _p );
 	void updateTopBottomLevels( void );
 
 	QScrollBar * m_leftRightScroll;
@@ -212,15 +204,18 @@ private:
 
 	actions m_action;
 
-	Uint32 m_selectStartTact64th;
-	int m_selectedTact64th;
-	int m_selectStartLevel;
-	int m_selectedLevels;
+	tick m_selectStartTick;
+	tick m_selectedTick;
+	float m_selectStartLevel;
+	float m_selectedLevels;
 
-	int m_moveStartLevel;
-	int m_moveStartTact64th;
+	float m_moveStartLevel;
+	tick m_moveStartTick;
 	int m_moveXOffset;
 
+	float m_drawLastLevel;
+	tick m_drawLastTick;
+
 	int m_ppt;
 	int m_y_delta;
 	bool m_y_auto;
