@@ -1,7 +1,7 @@
 /*
  * clipboard.h - the clipboard for patterns, notes etc.
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -26,35 +26,29 @@
 #ifndef _CLIPBOARD_H
 #define _CLIPBOARD_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include <QtCore/QMap>
+#include <QtXml/QDomElement>
 
 
-#include "qt3support.h"
+class journallingObject;
 
-#ifdef QT4
-
-#include <QMap>
-#include <Qt/QtXml>
-
-#else
-
-#include <qmap.h>
-#include <qdom.h>
-
-#endif
-
-
-class settings;
-
-namespace clipboard
+class clipboard
 {
+public:
 	typedef QMap<QString, QDomElement> map;
-	extern map content;
 
-	void FASTCALL copy( settings * _settings_object );
-	const QDomElement * FASTCALL getContent( const QString & _node_name );
+	static void copy( journallingObject * _object );
+	static const QDomElement * getContent( const QString & _node_name );
+
+	static const char * mimeType( void )
+	{
+		return( "application/x-lmms-clipboard" );
+	}
+
+
+private:
+	static map content;
+
 } ;
 
 #endif
