@@ -2,6 +2,7 @@
  * detuning_helper.h - detuning automation helper
  *
  * Copyright (c) 2007 Javier Serrano Polo <jasp00/at/users.sourceforge.net>
+ * Copyright (c) 2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,26 +27,37 @@
 #ifndef _DETUNING_HELPER_H
 #define _DETUNING_HELPER_H
 
-
-#include "shared_object.h"
+#include "inline_automation.h"
 
 
-class detuningHelper : public automatableObject<float>, public sharedObject
+class detuningHelper : public inlineAutomation
 {
 public:
 	detuningHelper( void ) :
-		autoObj( NULL )
+		inlineAutomation()
 	{
 	}
 
+	virtual ~detuningHelper()
+	{
+	}
+
+	virtual float defaultValue( void ) const
+	{
+		return( 0 );
+	}
+
 	virtual QString displayName( void ) const
 	{
-		return( automationEditor::tr( "Note detuning" ) );
+		return( tr( "Note detuning" ) );
+	}
+
+	inline virtual QString nodeName( void ) const
+	{
+		return( "detuning" );
 	}
 
 } ;
 
 
-
-
 #endif
