@@ -28,34 +28,41 @@
 #define _DUMMY_PLUGIN_H
 
 #include "plugin.h"
+#include "plugin_view.h"
 
 
 class dummyPlugin : public plugin
 {
 public:
-	dummyPlugin( void ) :
-		plugin( NULL )
+	inline dummyPlugin( void ) :
+		plugin( NULL, NULL )
 	{
 	}
 
-	virtual ~dummyPlugin()
+	inline virtual ~dummyPlugin()
 	{
 	}
 
 
-	virtual void FASTCALL saveSettings( QDomDocument &, QDomElement & )
+	inline virtual void saveSettings( QDomDocument &, QDomElement & )
 	{
 	}
 
-	virtual void FASTCALL loadSettings( const QDomElement & )
+	inline virtual void loadSettings( const QDomElement & )
 	{
 	}
 
-	virtual QString nodeName( void ) const
+	inline virtual QString nodeName( void ) const
 	{
 		return( "dummyplugin" );
 	}
 
+protected:
+	virtual pluginView * instantiateView( QWidget * _parent )
+	{
+		return( new pluginView( this, _parent ) );
+	}
+
 } ;
 
 
