@@ -1,7 +1,7 @@
 /*
  * engine.h - engine-system of LMMS
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,33 +26,30 @@
 #ifndef _ENGINE_H
 #define _ENGINE_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifndef QT3
+#include "lmmsconfig.h"
 
 #include <QtCore/QMap>
 
-#else
-
-#include <qmap.h>
-
-#endif
+#include "export.h"
 
 class automationEditor;
 class bbEditor;
+class bbTrackContainer;
+class dummyTrackContainer;
+class fxMixer;
+class fxMixerView;
 class projectJournal;
 class mainWindow;
 class mixer;
 class pianoRoll;
 class projectNotes;
+class song;
 class songEditor;
-#ifdef LADSPA_SUPPORT
 class ladspa2LMMS;
-#endif
+class controllerRackView;
 
-class engine
+
+class EXPORT engine
 {
 public:
 	static void init( const bool _has_gui = true );
@@ -63,16 +60,53 @@ public:
 		return( s_hasGUI );
 	}
 
+	static void setSuppressMessages( bool _on )
+	{
+		s_suppressMessages = _on;
+	}
+
+	static bool suppressMessages( void )
+	{
+		return !s_hasGUI || s_suppressMessages;
+	}
+
+	// core
 	static mixer * getMixer( void )
 	{
 		return( s_mixer );
 	}
 
+	static fxMixer * getFxMixer( void )
+	{
+		return( s_fxMixer );
+	}
+
+	static song * getSong( void )
+	{
+		return( s_song );
+	}
+
+	static bbTrackContainer * getBBTrackContainer( void )
+	{
+		return( s_bbTrackContainer );
+	}
+
+	static projectJournal * getProjectJournal( void )
+	{
+		return( s_projectJournal );
+	}
+
+	// GUI
 	static mainWindow * getMainWindow( void )
 	{
 		return( s_mainWindow );
 	}
 
+	static fxMixerView * getFxMixerView( void )
+	{
+		return( s_fxMixerView );
+	}
+
 	static songEditor * getSongEditor( void )
 	{
 		return( s_songEditor );
@@ -93,55 +127,65 @@ public:
 		return( s_projectNotes );
 	}
 
-	static projectJournal * getProjectJournal( void )
-	{
-		return( s_projectJournal );
-	}
-
 	static automationEditor * getAutomationEditor( void )
 	{
 		return( s_automationEditor );
 	}
 
-#ifdef LADSPA_SUPPORT
 	static ladspa2LMMS * getLADSPAManager( void )
 	{
 		return( s_ladspaManager );
 	}
-#endif
 
-	static float framesPerTact64th( void )
+	static dummyTrackContainer * getDummyTrackContainer( void )
 	{
-		return( s_frames_per_tact64th );
+		return( s_dummyTC );
 	}
-	static void updateFramesPerTact64th( void );
 
-	static const QMap<QString, QString> & sampleExtensions( void )
+	static controllerRackView * getControllerRackView( void )
 	{
-		return( s_sample_extensions );
+		return( s_controllerRackView );
+	}
+
+	static float framesPerTick( void )
+	{
+		return( s_framesPerTick );
+	}
+	static void updateFramesPerTick( void );
+
+	static const QMap<QString, QString> & pluginFileHandling( void )
+	{
+		return( s_pluginFileHandling );
 	}
 
 
 private:
 	static bool s_hasGUI;
-	static float s_frames_per_tact64th;
+	static bool s_suppressMessages;
+	static float s_framesPerTick;
 
+	// core
 	static mixer * s_mixer;
+	static fxMixer * s_fxMixer;
+	static song * s_song;
+	static bbTrackContainer * s_bbTrackContainer;
+	static projectJournal * s_projectJournal;
+	static dummyTrackContainer * s_dummyTC;
+	static controllerRackView * s_controllerRackView;
+
+	// GUI
 	static mainWindow * s_mainWindow;
+	static fxMixerView * s_fxMixerView;
 	static songEditor * s_songEditor;
 	static automationEditor * s_automationEditor;
 	static bbEditor * s_bbEditor;
 	static pianoRoll * s_pianoRoll;
 	static projectNotes * s_projectNotes;
-	static projectJournal * s_projectJournal;
-	
-#ifdef LADSPA_SUPPORT
 	static ladspa2LMMS * s_ladspaManager;
-#endif
 
-	static QMap<QString, QString> s_sample_extensions;
+	static QMap<QString, QString> s_pluginFileHandling;
 
-	static void load_extensions( void );
+	static void initPluginFileHandling( void );
 
 } ;
 
