@@ -26,41 +26,19 @@
 #ifndef _FILE_BROWSER_H
 #define _FILE_BROWSER_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QDir>
 #include <QtCore/QMutex>
-
-#else
-
-#include <qlistview.h>
-#include <qdir.h>
-#include <qmutex.h>
-
-#define cleanPath cleanDirPath
-
-#endif
+#include <QtGui/QTreeWidget>
 
 
 #include "side_bar_widget.h"
 
 
-#ifndef QT3
-class QListViewItem;
-#else
-class Q3ListViewItem;
-#endif
-class QPixmap;
+class QLineEdit;
 
 class fileItem;
-class listView;
+class instrumentTrack;
+class fileBrowserTreeWidget;
 class playHandle;
 class trackContainer;
 
@@ -72,91 +50,83 @@ class fileBrowser : public sideBarWidget
 public:
 	fileBrowser( const QString & _directories, const QString & _filter,
 			const QString & _title, const QPixmap & _pm,
-							QWidget * _parent );
+			QWidget * _parent, bool _dirs_as_items = false );
 	virtual ~fileBrowser();
 
-	static bool isDirWithContent( const QString & _path,
-						const QString & _filter );
-
 
 public slots:
+	void filterItems( const QString & _filter );
 	void reloadTree( void );
 
 
-protected slots:
-	void contextMenuRequest( Q3ListViewItem * _i, const QPoint & _pos,
-								int _col );
-	void contextMenuRequest( QListViewItem * _i, const QPoint & _pos,
-								int _col );
-	void sendToActiveInstrumentTrack( void );
-	void openInNewInstrumentTrackSE( void );
-	void openInNewInstrumentTrackBBE( void );
-
-
 private:
+	bool filterItems( QTreeWidgetItem * _item, const QString & _filter );
 	virtual void keyPressEvent( QKeyEvent * _ke );
 
 	void addItems( const QString & _path );
-	void openInNewInstrumentTrack( trackContainer * _tc );
 
-	listView * m_l;
-	fileItem * m_contextMenuItem;
+	fileBrowserTreeWidget * m_l;
+
+	QLineEdit * m_filterEdit;
 
 	QString m_directories;
 	QString m_filter;
 
+	bool m_dirsAsItems;
 
 } ;
 
 
 
 
-class listView : public Q3ListView
+class fileBrowserTreeWidget : public QTreeWidget
 {
 	Q_OBJECT
 public:
-	listView( QWidget * _parent );
-	virtual ~listView();
+	fileBrowserTreeWidget( QWidget * _parent );
+	virtual ~fileBrowserTreeWidget();
 
 
 protected:
-	virtual void contentsMouseDoubleClickEvent( QMouseEvent * _me );
-	virtual void contentsMousePressEvent( QMouseEvent * _me );
-	virtual void contentsMouseMoveEvent( QMouseEvent * _me );
-	virtual void contentsMouseReleaseEvent( QMouseEvent * _me );
+	virtual void contextMenuEvent( QContextMenuEvent * _e );
+	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void mouseMoveEvent( QMouseEvent * _me );
+	virtual void mouseReleaseEvent( QMouseEvent * _me );
 
 
 private:
+	void handleFile( fileItem * _fi, instrumentTrack * _it );
+	void openInNewInstrumentTrack( trackContainer * _tc );
+
+
 	bool m_mousePressed;
 	QPoint m_pressPos;
 
 	playHandle * m_previewPlayHandle;
 	QMutex m_pphMutex;
 
+	fileItem * m_contextMenuItem;
+
+
+private slots:
+	void activateListItem( QTreeWidgetItem * _item, int _column );
+	void openInNewInstrumentTrackBBE( void );
+	void openInNewInstrumentTrackSE( void );
+	void sendToActiveInstrumentTrack( void );
+	void updateDirectory( QTreeWidgetItem * _item );
+
 } ;
 
 
 
 
-class directory :
-#ifndef QT3
-	public Q3ListViewItem
-// trick moc
-#if 0
-		,
-#endif
-#else
-	public QListViewItem
-#endif
+class directory : public QTreeWidgetItem
 {
 public:
-	directory( Q3ListView * _parent, const QString & _filename,
-			const QString & _path, const QString & _filter );
-	directory( directory * _parent, const QString & _filename,
-			const QString & _path, const QString & _filter );
+	directory( const QString & _filename, const QString & _path,
+						const QString & _filter );
 
-	void setOpen( bool );
-	void setup( void );
+	void update( void );
 
 	inline QString fullName( QString _path = QString::null )
 	{
@@ -164,16 +134,14 @@ public:
 		{
 			_path = m_directories[0];
 		}
-		return( QDir::cleanPath( _path + QDir::separator() +
-								text( 0 ) ) +
+		if( _path != QString::null )
+		{
+			_path += QDir::separator();
+		}
+		return( QDir::cleanPath( _path + text( 0 ) ) +
 							QDir::separator() );
 	}
 
-	inline const QPixmap * pixmap( int ) const
-	{
-		return( m_pix );
-	}
-
 	inline void addDirectory( const QString & _dir )
 	{
 		m_directories.push_back( _dir );
@@ -181,8 +149,7 @@ public:
 
 
 private:
-	void initPixmapStuff( void );
-	void FASTCALL setPixmap( const QPixmap * _px );
+	void initPixmaps( void );
 
 	bool addItems( const QString & _path );
 
@@ -191,8 +158,6 @@ private:
 	static QPixmap * s_folderOpenedPixmap;
 	static QPixmap * s_folderLockedPixmap;
 
-	directory * m_p;
-	const QPixmap * m_pix;
 	QStringList m_directories;
 	QString m_filter;
 
@@ -201,55 +166,58 @@ private:
 
 
 
-class fileItem :
-#ifndef QT3
-	public Q3ListViewItem
-// trick moc
-#if 0
-		,
-#endif
-#else
-	public QListViewItem
-#endif
+class fileItem : public QTreeWidgetItem
 {
 public:
-	fileItem( Q3ListView * _parent, const QString & _name,
-							const QString & _path );
-#ifndef QT3
-	fileItem( Q3ListViewItem * _parent, const QString & _name,
-							const QString & _path );
-#else
-	fileItem( QListViewItem * _parent, const QString & _name,
-							const QString & _path );
-#endif
+	enum FileTypes
+	{
+		ProjectFile,
+		PresetFile,
+		SampleFile,
+		SoundFontFile,
+		PatchFile,
+		MidiFile,
+		FlpFile,
+		UnknownFile,
+		NumFileTypes
+	} ;
 
-	QString fullName( void ) const
+	enum FileHandling
+	{
+		NotSupported,
+		LoadAsProject,
+		LoadAsPreset,
+		LoadByPlugin,
+		ImportAsProject
+	} ;
+
+
+	fileItem( QTreeWidget * _parent, const QString & _name,
+							const QString & _path );
+	fileItem( const QString & _name, const QString & _path );
+
+	inline QString fullName( void ) const
 	{
 		return( QDir::cleanPath( m_path ) + QDir::separator() +
 								text( 0 ) );
 	}
-	inline const QPixmap * pixmap( int ) const
-	{
-		return( m_pix );
-	}
 
-	enum fileTypes
-	{
-		PROJECT_FILE, PRESET_FILE, SAMPLE_FILE, MIDI_FILE, FLP_FILE,
-		UNKNOWN
-	} ;
-	
-	inline fileTypes type( void )
+	inline FileTypes type( void ) const
 	{
 		return( m_type );
 	}
 
+	inline FileHandling handling( void ) const
+	{
+		return( m_handling );
+	}
+
 	QString extension( void );
 	static QString extension( const QString & _file );
 
 
 private:
-	void initPixmapStuff( void );
+	void initPixmaps( void );
 	void determineFileType( void );
 
 	static QPixmap * s_projectFilePixmap;
@@ -258,16 +226,12 @@ private:
 	static QPixmap * s_midiFilePixmap;
 	static QPixmap * s_flpFilePixmap;
 	static QPixmap * s_unknownFilePixmap;
-	
-	QPixmap * m_pix;
+
 	QString m_path;
-	fileTypes m_type;
+	FileTypes m_type;
+	FileHandling m_handling;
+
 } ;
 
 
-#ifdef QT3
-#undef cleanPath
-#endif
-
-
 #endif
