@@ -1,7 +1,7 @@
 /*
  * group_box.h - LMMS-groupbox
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,66 +26,45 @@
 #ifndef _GROUP_BOX_H
 #define _GROUP_BOX_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QWidget>
 
-#else
-
-#include <qwidget.h>
-
-#endif
-
+#include "automatable_model_view.h"
 #include "pixmap_button.h"
 
 
 class QPixmap;
 
 
-class groupBox : public QWidget
+class groupBox : public QWidget, public boolModelView
 {
 	Q_OBJECT
 public:
-	groupBox( const QString & _caption, QWidget * _parent, track * _track );
+	groupBox( const QString & _caption, QWidget * _parent );
 	virtual ~groupBox();
 
-	bool isActive( void ) const
+	virtual void modelChanged( void );
+
+	pixmapButton * ledButton( void )
 	{
-		return( m_led->isChecked() );
+		return m_led;
 	}
 
-	void saveSettings( QDomDocument & _doc, QDomElement & _this,
-							const QString & _name );
-	void loadSettings( const QDomElement & _this, const QString & _name );
-
-
-public slots:
-	void setState( bool _on, bool _anim = FALSE );
-	void animate( void );
-
-
-signals:
-	void toggled( bool _state );
-
 
 protected:
-	virtual void resizeEvent( QResizeEvent * _re );
 	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void resizeEvent( QResizeEvent * _re );
 
 
 private:
 	void updatePixmap( void );
 
-	static QPixmap * s_ledBg;
-
 	pixmapButton * m_led;
 	QString m_caption;
 
-	int m_origHeight;
-	bool m_animating;
-
 } ;
 
+
+typedef boolModel groupBoxModel;
+
+
 #endif
