@@ -2,7 +2,7 @@
  * import_filter.h - declaration of class importFilter, the base-class for all
  *                   file import filters
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,23 +27,8 @@
 #ifndef _IMPORT_FILTER_H
 #define _IMPORT_FILTER_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QFile>
 
-#else
-
-#include <qfile.h>
-
-#endif
-
 
 #include "plugin.h"
 
@@ -51,7 +36,7 @@
 class trackContainer;
 
 
-class importFilter : public plugin
+class EXPORT importFilter : public plugin
 {
 public:
 	importFilter( const QString & _file_name,
@@ -61,7 +46,7 @@ public:
 
 	// tries to import given file to given track-container by having all
 	// available import-filters to try to import the file
-	static void FASTCALL import( const QString & _file_to_import,
+	static void import( const QString & _file_to_import,
 						trackContainer * _tc );
 
 
@@ -82,42 +67,29 @@ protected:
 
 	inline int readByte( void )
 	{
-#ifdef QT4
-		char c;
-		if( m_file.getChar( &c ) )
+		unsigned char c;
+		if( m_file.getChar( (char*) &c ) )
 		{
 			return( static_cast<int>( c ) );
 		}
 		return( -1 );
-#else
-		return( m_file.getch() );
-#endif
 	}
 
 	inline int readBlock( char * _data, int _len )
 	{
-#ifndef QT3
 		return( m_file.read( _data, _len ) );
-#else
-		return( m_file.readBlock( _data, _len ) );
-#endif
 	}
 
-	inline void ungetChar( int _ch )
+	inline void ungetChar( char _ch )
 	{
-#ifndef QT3
 		m_file.ungetChar( _ch );
-#else
-		m_file.ungetch( _ch );
-#endif
 	}
 
-	virtual void FASTCALL saveSettings( QDomDocument &,
-						QDomElement & )
+	virtual void saveSettings( QDomDocument &, QDomElement & )
 	{
 	}
 
-	virtual void FASTCALL loadSettings( const QDomElement & )
+	virtual void loadSettings( const QDomElement & )
 	{
 	}
 
