@@ -1,7 +1,7 @@
 /*
  * midi_alsa_raw.h - midi-client for RawMIDI via ALSA
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,31 +26,14 @@
 #ifndef _MIDI_ALSA_RAW_H
 #define _MIDI_ALSA_RAW_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "lmmsconfig.h"
 
-#ifdef HAVE_ALSA_ASOUNDLIB_H
-
-#ifndef ALSA_SUPPORT
-#define ALSA_SUPPORT
-#endif
+#ifdef LMMS_HAVE_ALSA
 
 #include <alsa/asoundlib.h>
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QThread>
 
-#else
-
-#include <qthread.h>
-
-#endif
-
-
 #include "midi_client.h"
 
 
@@ -89,7 +72,7 @@ public:
 
 
 protected:
-	virtual void FASTCALL sendByte( const Uint8 _c );
+	virtual void sendByte( const Uint8 _c );
 	virtual void run( void );
 
 
