@@ -1,7 +1,7 @@
 /*
  * midi_event_processor.h - base-class for midi-processing classes
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,11 +26,6 @@
 #ifndef _MIDI_EVENT_PROCESSOR_H
 #define _MIDI_EVENT_PROCESSOR_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-
 class midiEvent;
 class midiTime;
 
@@ -48,9 +43,9 @@ public:
 	}
 
 	// to be implemented by inheriting classes
-	virtual void FASTCALL processInEvent( const midiEvent & _me,
+	virtual void processInEvent( const midiEvent & _me,
 						const midiTime & _time ) = 0;
-	virtual void FASTCALL processOutEvent( const midiEvent & _me,
+	virtual void processOutEvent( const midiEvent & _me,
 						const midiTime & _time ) = 0;
 
 } ;
