@@ -27,107 +27,126 @@
 #ifndef _MIDI_TIME_H
 #define _MIDI_TIME_H
 
-#include "types.h"
+#include "lmms_basics.h"
+#include "export.h"
+
+const int DefaultTicksPerTact = 192;
+const int DefaultStepsPerTact = 16;
+const int DefaultBeatsPerTact = DefaultTicksPerTact / DefaultStepsPerTact;
 
 
-class midiTime
+class EXPORT midiTime
 {
 public:
-	inline midiTime( const tact _tact, const tact64th _tact_64th ) :
-		m_tact( _tact ),
-		m_tact64th( _tact_64th )
+	inline midiTime( const tact _tact, const tick _ticks ) :
+		m_ticks( _tact * s_ticksPerTact + _ticks )
 	{
 	}
 
-	midiTime( const Sint32 _abs = 0 ) :
-		m_tact( _abs / 64 ),
-		m_tact64th( _abs % 64 )
+	inline midiTime( const tick _ticks = 0 ) :
+		m_ticks( _ticks )
 	{
 	}
 
-	inline midiTime( const midiTime & _t )
+	inline midiTime( const midiTime & _t ) :
+		m_ticks( _t.m_ticks )
 	{
-		*this = _t;
 	}
 
-	midiTime toNearestTact( void ) const
+	inline midiTime toNearestTact( void ) const
 	{
-		if( m_tact64th >= 32 )
+		if( m_ticks % s_ticksPerTact >= s_ticksPerTact/2 )
 		{
-			return( m_tact * 64 + 64 );
+			return( ( getTact() + 1 ) * s_ticksPerTact );
 		}
-		return( m_tact * 64 );
+		return( getTact() * s_ticksPerTact );
 	}
 
 	inline midiTime & operator=( const midiTime & _t )
 	{
-		m_tact = _t.m_tact;
-		m_tact64th = _t.m_tact64th;
+		m_ticks = _t.m_ticks;
 		return( *this );
 	}
 
 	inline midiTime & operator+=( const midiTime & _t )
 	{
-		return( *this = static_cast<Sint32>( *this ) +
-						static_cast<Sint32>( _t ) );
+		m_ticks += _t.m_ticks;
+		return *this;
 	}
 
 	inline midiTime & operator-=( const midiTime & _t )
 	{
-		return( *this = static_cast<Sint32>( *this ) -
-						static_cast<Sint32>( _t ) );
-	}
-
-	inline void setTact( tact _t )
-	{
-		m_tact = _t;
+		m_ticks -= _t.m_ticks;
+		return *this;
 	}
 
 	inline tact getTact( void ) const
 	{
-		return( m_tact );
+		return( m_ticks / s_ticksPerTact );
 	}
 
-	inline void setTact64th( tact64th _t )
+	inline tact nextFullTact( void ) const
 	{
-		m_tact64th = _t;
+		if( m_ticks % s_ticksPerTact == 0 )
+		{
+			return( m_ticks / s_ticksPerTact );
+		}
+		return( m_ticks / s_ticksPerTact + 1 );
 	}
 
-	inline tact64th getTact64th( void ) const
+	inline void setTicks( tick _t )
 	{
-		return( m_tact64th );
+		m_ticks = _t;
 	}
 
-	// converts time-class in an absolute value, useful for calculations,
-	// comparisons and so on...
-	inline operator Sint32( void ) const
+	inline tick getTicks( void ) const
 	{
-		return( static_cast<Sint32>( m_tact ) * 64 +
-					static_cast<Sint32>( m_tact64th ) );
+		return( m_ticks );
+	}
+
+	inline operator int( void ) const
+	{
+		return( m_ticks );
 	}
 
 	// calculate number of frame that are needed this time
-	inline f_cnt_t frames( const float _frames_per_tact64th ) const
+	inline f_cnt_t frames( const float _frames_per_tick ) const
 	{
-		if( m_tact >= 0 )
+		if( m_ticks >= 0 )
 		{
-			return( static_cast<f_cnt_t>( ( m_tact * 64
-				+ m_tact64th ) * _frames_per_tact64th ) );
+			return( static_cast<f_cnt_t>( m_ticks *
+							_frames_per_tick ) );
 		}
 		return( 0 );
 	}
 
 	static inline midiTime fromFrames( const f_cnt_t _frames,
-					const float _frames_per_tact64th )
+					const float _frames_per_tick )
 	{
-		return( midiTime( static_cast<Sint32>( _frames /
-						_frames_per_tact64th ) ) );
+		return( midiTime( static_cast<int>( _frames /
+							_frames_per_tick ) ) );
 	}
 
 
+	static tick ticksPerTact( void )
+	{
+		return( s_ticksPerTact );
+	}
+
+	static int stepsPerTact( void )
+	{
+		return( ticksPerTact() / DefaultBeatsPerTact );
+	}
+
+	static void setTicksPerTact( tick _tpt )
+	{
+		s_ticksPerTact = _tpt;
+	}
+
 private:
-	tact m_tact;
-	tact64th m_tact64th;
+	tick m_ticks;
+
+	static tick s_ticksPerTact;
 
 } ;
 
