@@ -2,7 +2,7 @@
  * note_play_handle.h - declaration of class notePlayHandle which is needed
  *                      by LMMS-Play-Engine
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,9 +27,8 @@
 #ifndef _NOTE_PLAY_HANDLE_H
 #define _NOTE_PLAY_HANDLE_H
 
-#include "play_handle.h"
-#include "basic_filters.h"
-#include "bb_track.h"
+#include "lmmsconfig.h"
+#include "mixer.h"
 #include "note.h"
 #include "instrument.h"
 #include "instrument_track.h"
@@ -38,11 +37,12 @@
 
 class notePlayHandle;
 
-typedef vvector<notePlayHandle *> notePlayHandleVector;
-typedef vvector<const notePlayHandle *> constNotePlayHandleVector;
+template<ch_cnt_t=DEFAULT_CHANNELS> class basicFilters;
+typedef QVector<notePlayHandle *> notePlayHandleVector;
+typedef QVector<const notePlayHandle *> constNotePlayHandleVector;
 
 
-class notePlayHandle : public playHandle, public note
+class EXPORT notePlayHandle : public playHandle, public note
 {
 public:
 	void * m_pluginData;
@@ -52,18 +52,27 @@ public:
 					const f_cnt_t _offset,
 					const f_cnt_t _frames, const note & _n,
 					notePlayHandle * _parent = NULL,
-					const bool _arp_note = FALSE );
+					const bool _part_of_arp = false );
 	virtual ~notePlayHandle();
 
+	virtual void setVolume( const volume _volume = DefaultVolume );
 
-	const float & frequency( void )
+	int getMidiVelocity( void ) const;
+
+	const float & frequency( void ) const
 	{
 		return( m_frequency );
 	}
 
 	void updateFrequency( void );
 
-	virtual void play( bool _try_parallelizing );
+	// returns frequency without pitch-wheel influence
+	float unpitchedFrequency( void ) const
+	{
+		return( m_unpitchedFrequency );
+	}
+
+	virtual void play( sampleFrame * _working_buffer );
 
 	virtual inline bool done( void ) const
 	{
@@ -78,7 +87,7 @@ public:
 		if( m_released == TRUE )
 		{
 			f_cnt_t todo = engine::getMixer()->framesPerPeriod();
-			if( arpBaseNote() == TRUE )
+			if( isArpeggioBaseNote() )
 			{
 				rftd = rfd + 2 *
 					engine::getMixer()->framesPerPeriod();
@@ -111,7 +120,7 @@ public:
 			}
 		}
 
-		if( arpBaseNote() == TRUE && m_subNotes.size() == 0 )
+		if( isArpeggioBaseNote() && m_subNotes.size() == 0 )
 		{
 			rfd = rftd;
 		}
@@ -121,9 +130,9 @@ public:
 
 	f_cnt_t framesLeft( void ) const;
 
-	inline f_cnt_t framesLeftForCurrentPeriod( void ) const
+	inline fpp_t framesLeftForCurrentPeriod( void ) const
 	{
-		return( tMin<f_cnt_t>( framesLeft(),
+		return( (fpp_t) qMin<f_cnt_t>( framesLeft(),
 				engine::getMixer()->framesPerPeriod() ) );
 	}
 
@@ -131,7 +140,7 @@ public:
 	virtual bool isFromTrack( const track * _track ) const;
 
 
-	void FASTCALL noteOff( const f_cnt_t _s = 0 );
+	void noteOff( const f_cnt_t _s = 0 );
 
 	inline f_cnt_t framesBeforeRelease( void ) const
 	{
@@ -167,7 +176,7 @@ public:
 	}
 
 	// returns volume-level at frame _frame (envelope/LFO)
-	float FASTCALL volumeLevel( const f_cnt_t _frame );
+	float volumeLevel( const f_cnt_t _frame );
 
 	// returns instrument-track this note-play-handle plays
 	inline instrumentTrack * getInstrumentTrack( void )
@@ -177,29 +186,29 @@ public:
 
 	// returns whether note is a base-note, e.g. is not part of an arpeggio
 	// or a chord
-	inline bool baseNote( void ) const
+	inline bool isBaseNote( void ) const
 	{
 		return( m_baseNote );
 	}
 
-	// returns whether note is part of an arpeggio
-	inline bool arpNote( void ) const
+	inline bool isPartOfArpeggio( void ) const
 	{
-		return( m_arpNote );
+		return( m_partOfArpeggio );
 	}
 
-	inline void setArpNote( const bool _on )
+	inline void setPartOfArpeggio( const bool _on )
 	{
-		m_arpNote = _on;
+		m_partOfArpeggio = _on;
 	}
 
 	// returns whether note is base-note for arpeggio
-	inline bool arpBaseNote( void ) const
+	inline bool isArpeggioBaseNote( void ) const
 	{
-		return( baseNote() && arpNote() );
+		return( isBaseNote() && ( m_partOfArpeggio || 
+				m_instrumentTrack->arpeggiatorEnabled() ) );
 	}
 
-	inline bool muted( void ) const
+	inline bool isMuted( void ) const
 	{
 		return( m_muted );
 	}
@@ -207,7 +216,7 @@ public:
 	void mute( void );
 
 	// returns index of note-play-handle in vector of note-play-handles 
-	// belonging to this channel
+	// belonging to this instrument-track - used by arpeggiator
 	int index( void ) const;
 
 	// note-play-handles belonging to given channel, if _all_ph = TRUE,
@@ -220,37 +229,17 @@ public:
 
 	bool bbTrackMuted( void )
 	{
-		return( m_bbTrack && m_bbTrack->muted() );
+		return( m_bbTrack && m_bbTrack->isMuted() );
 	}
-	void setBBTrack( bbTrack * _bb_track )
+	void setBBTrack( track * _bb_track )
 	{
 		m_bbTrack = _bb_track;
 	}
 
-
-	virtual bool supportsParallelizing( void ) const
-	{
-		return( m_instrumentTrack->getInstrument()->
-						supportsParallelizing()
-				&&
-			// we must not parallelize note-play-handles, which
-			// belong to instruments that are instrument-play-
-			// handle-driven, because then waitForWorkerThread()
-			// would be additionally called for each
-			// note-play-handle which results in hangups
-			m_instrumentTrack->getInstrument()->
-						notePlayHandleBased() );
-	}
-
-	virtual void waitForWorkerThread( void )
-	{
-		m_instrumentTrack->m_instrument->waitForWorkerThread();
-	}
-
 	void processMidiTime( const midiTime & _time );
-	void resize( const bpm_t _new_bpm );
+	void resize( const bpm_t _new_tempo );
 
-#if SINGERBOT_SUPPORT
+#ifdef LMMS_SINGERBOT_SUPPORT
 	int patternIndex( void )
 	{
 		return( m_patternIndex );
@@ -269,23 +258,20 @@ private:
 	public:
 		baseDetuning( detuningHelper * _detuning );
 
-		int level( void )
+		void setValue( float _val )
 		{
-			return( m_level );
+			m_value = _val;
 		}
 
-		void setLevel( int _level );
-
-		float value( void )
+		float value( void ) const
 		{
 			return( m_value );
 		}
 
 
 	private:
-		int m_level;
-		float m_value;
 		detuningHelper * m_detuning;
+		float m_value;
 
 	} ;
 
@@ -303,25 +289,26 @@ private:
 	f_cnt_t m_releaseFramesDone;	// number of frames done after
 					// release of note
 	notePlayHandleVector m_subNotes;// used for chords and arpeggios
-	bool m_released;		// indicates whether note is released
+	volatile bool m_released;	// indicates whether note is released
 	bool m_baseNote;		// indicates whether note is a
 					// base-note (i.e. no sub-note)
-	bool m_arpNote;			// indicates whether note is part of
+	bool m_partOfArpeggio;		// indicates whether note is part of
 					// an arpeggio (either base-note or
 					// sub-note)
 	bool m_muted;			// indicates whether note is muted
-	bbTrack * m_bbTrack;		// related BB track
-#if SINGERBOT_SUPPORT
+	track * m_bbTrack;		// related BB track
+#ifdef LMMS_SINGERBOT_SUPPORT
 	int m_patternIndex;		// position among relevant notes
 #endif
 
 	// tempo reaction
-	bpm_t m_orig_bpm;		// original bpm
-	f_cnt_t m_orig_frames;		// original m_frames
+	bpm_t m_origTempo;		// original tempo
+	f_cnt_t m_origFrames;		// original m_frames
 
 	float m_frequency;
+	float m_unpitchedFrequency;
 
-	baseDetuning * m_base_detuning;
+	baseDetuning * m_baseDetuning;
 
 } ;
 
