@@ -26,26 +26,10 @@
 #ifndef _NSTATE_BUTTON_H
 #define _NSTATE_BUTTON_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QPixmap>
 #include <QtCore/QVector>
 #include <QtCore/QPair>
 
-#else
-
-#include <qpixmap.h>
-#include <qvaluevector.h>
-#include <qpair.h>
-
-#endif
-
 #include "tool_button.h"
 
 
@@ -81,7 +65,7 @@ protected:
 
 
 private:
-	vvector<QPair<QPixmap, QString> > m_states;
+	QVector<QPair<QPixmap, QString> > m_states;
 	QString m_generalToolTip;
 
 	int m_curState;
