@@ -1,8 +1,8 @@
 /*
- * panning.h - declaration of some constants and types, concerning the
+ * panning.h - declaration of some types, concerning the
  *             panning of a note
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,11 +26,27 @@
 #ifndef _PANNING_H
 #define _PANNING_H
 
-#include "types.h"
+#include "lmms_basics.h"
+#include "volume.h"
+#include "templates.h"
+#include "panning_constants.h"
+#include "midi.h"
 
-const panning PANNING_RIGHT = ( 0 + 100);
-const panning PANNING_LEFT = - PANNING_RIGHT;
-const panning PANNING_CENTER = 0;
-const panning DEFAULT_PANNING = PANNING_CENTER;
+inline stereoVolumeVector panningToVolumeVector( panning _p,
+							float _scale = 1.0f )
+{
+	stereoVolumeVector v = { { _scale, _scale } };
+	const float pf = _p / 100.0f;
+	v.vol[_p >= PanningCenter ? 0 : 1] *= 1.0f - qAbs<float>( pf );
+	return v;
+}
+
+inline Sint16 panningToMidi( panning _p )
+{
+	return MidiMinPanning + 
+			  ( (float)( _p - PanningLeft ) ) / 
+			  ( (float)( PanningRight - PanningLeft ) ) *
+			  ( (float)( MidiMaxPanning - MidiMinPanning ) );
+}
 
 #endif
