@@ -1,7 +1,7 @@
 /*
  * pixmap_button.h - declaration of class pixmapButton
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,50 +26,38 @@
 #ifndef _PIXMAP_BUTTON_H
 #define _PIXMAP_BUTTON_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QPixmap>
 
-#else
-
-#include <qpixmap.h>
-
-#endif
-
-
 #include "automatable_button.h"
 
 
-class pixmapButton : public automatableButton
+class EXPORT pixmapButton : public automatableButton
 {
 	Q_OBJECT
 public:
-	pixmapButton( QWidget * _parent, const QString & _name,
-							track * _track );
+	pixmapButton( QWidget * _parent,
+					const QString & _name = QString::null );
 	virtual ~pixmapButton();
 
 	void setActiveGraphic( const QPixmap & _pm );
 	void setInactiveGraphic( const QPixmap & _pm, bool _update = TRUE );
-	void setBgGraphic( const QPixmap & _pm );
 
 
 signals:
 	void doubleClicked( void );
-	void clickedRight( void );
 
 
 protected:
 	virtual void paintEvent( QPaintEvent * _pe );
 	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void mouseReleaseEvent( QMouseEvent * _me );
 	virtual void mouseDoubleClickEvent( QMouseEvent * _me );
 
 
 private:
 	QPixmap m_activePixmap;
 	QPixmap m_inactivePixmap;
-	QPixmap m_bgPixmap;
+	bool	m_pressed;
 
 } ;
 
