@@ -1,8 +1,7 @@
 /*
- * play_handle.h - base-class playHandle which is needed by
- *                 LMMS-Player-Engine
+ * play_handle.h - base-class playHandle - core of rendering engine
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,23 +26,10 @@
 #ifndef _PLAY_HANDLE_H
 #define _PLAY_HANDLE_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include "qt3support.h"
-
-#ifdef QT4
-
+#include <QtCore/QThread>
 #include <QtCore/QVector>
 
-#else
-
-#include <qvaluevector.h>
-
-#endif
-
-#include "types.h"
+#include "lmms_basics.h"
 
 class track;
 
@@ -61,7 +47,8 @@ public:
 
 	playHandle( const types _type, f_cnt_t _offset = 0 ) :
 		m_type( _type ),
-		m_offset( _offset )
+		m_offset( _offset ),
+		m_affinity( QThread::currentThread() )
 	{
 	}
 
@@ -69,19 +56,29 @@ public:
 	{
 	}
 
-	inline types type( void ) const
+	virtual inline bool affinityMatters( void ) const
 	{
-		return( m_type );
+		return false;
 	}
 
-	virtual void play( bool _try_parallelizing = FALSE ) = 0;
+	const QThread * affinity( void ) const
+	{
+		return m_affinity;
+	}
+
+	inline types type( void ) const
+	{
+		return m_type;
+	}
+
+	virtual void play( sampleFrame * _working_buffer ) = 0;
 	virtual bool done( void ) const = 0;
 
 	// returns how many frames this play-handle is aligned ahead, i.e.
 	// at which position it is inserted in the according buffer
 	inline f_cnt_t offset( void ) const
 	{
-		return ( m_offset );
+		return m_offset;
 	}
 
 	inline void setOffset( f_cnt_t _offset )
@@ -92,25 +89,17 @@ public:
 
 	virtual bool isFromTrack( const track * _track ) const = 0;
 
-	virtual bool supportsParallelizing( void ) const
-	{
-		return( FALSE );
-	}
-
-	virtual void waitForWorkerThread( void )
-	{
-	}
-
 
 private:
 	types m_type;
 	f_cnt_t m_offset;
+	const QThread * m_affinity;
 
 } ;
 
 
-typedef vvector<playHandle *> playHandleVector;
-typedef vvector<const playHandle *> constPlayHandleVector;
+typedef QVector<playHandle *> playHandleVector;
+typedef QVector<const playHandle *> constPlayHandleVector;
 
 
 #endif
