@@ -1,8 +1,8 @@
 /*
  * preset_preview_play_handle.h - play-handle for playing a short preview-sound
- *                                of a preset
+ *                                of a preset or a file processed by a plugin
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,9 +27,6 @@
 #ifndef _PRESET_PREVIEW_PLAY_HANDLE_H
 #define _PRESET_PREVIEW_PLAY_HANDLE_H
 
-#include "qt3support.h"
-
-
 #include "note_play_handle.h"
 
 
@@ -40,16 +37,17 @@ class previewTrackContainer;
 class presetPreviewPlayHandle : public playHandle
 {
 public:
-	presetPreviewPlayHandle( const QString & _preset_file );
+	presetPreviewPlayHandle( const QString & _preset_file,
+						bool _load_by_plugin = false );
 	virtual ~presetPreviewPlayHandle();
 
-	virtual void play( bool _try_parallelizing );
+	virtual void play( sampleFrame * _working_buffer );
 	virtual bool done( void ) const;
 
 	virtual bool isFromTrack( const track * _track ) const;
 
 	static void init( void );
-	static void cleanUp( void );
+	static void cleanup( void );
 	static constNotePlayHandleVector nphsOfInstrumentTrack(
 						const instrumentTrack * _ct );
 
