@@ -1,7 +1,7 @@
 /*
  * project_journal.h - declaration of class projectJournal
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,22 +26,11 @@
 #ifndef _PROJECT_JOURNAL_H
 #define _PROJECT_JOURNAL_H
 
-#include "types.h"
-#include "qt3support.h"
-
-#ifndef QT3
-
 #include <QtCore/QHash>
 #include <QtCore/QVariant>
 #include <QtCore/QVector>
 
-#else
-
-#include <qvariant.h>
-#include <qvaluevector.h>
-#include <qmap.h>
-
-#endif
+#include "lmms_basics.h"
 
 
 class journallingObject;
@@ -89,13 +78,7 @@ public:
 	// journalling information about the ID get's lost
 	void forgetAboutID( const jo_id_t _id );
 
-	void clearJournal( void )
-	{
-		m_journalEntries.clear();
-		m_currentJournalEntry = m_journalEntries.end();
-	}
-
-	void clearInvalidJournallingObjects( void );
+	void clearJournal( void );
 
 	journallingObject * getJournallingObject( const jo_id_t _id )
 	{
@@ -108,12 +91,8 @@ public:
 
 
 private:
-#ifndef QT3
 	typedef QHash<jo_id_t, journallingObject *> joIDMap;
-#else
-	typedef QMap<jo_id_t, journallingObject *> joIDMap;
-#endif
-	typedef vvector<jo_id_t> journalEntryVector;
+	typedef QVector<jo_id_t> journalEntryVector;
 
 	joIDMap m_joIDs;
 
