@@ -1,7 +1,7 @@
 /*
  * setup_dialog.h - dialog for setting up LMMS
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,21 +26,10 @@
 #ifndef _SETUP_DIALOG_H
 #define _SETUP_DIALOG_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QDialog>
 #include <QtCore/QMap>
 
-#else
-
-#include <qdialog.h>
-#include <qmap.h>
-
-#endif
-
-
+#include "lmmsconfig.h"
 #include "audio_device.h"
 #include "midi_client.h"
 
@@ -57,13 +46,16 @@ class setupDialog : public QDialog
 {
 	Q_OBJECT
 public:
-	enum configTabs
+	enum ConfigTabs
 	{
-		GENERAL_SETTINGS, DIRECTORY_SETTINGS, PERFORMANCE_SETTINGS,
-		AUDIO_SETTINGS, MIDI_SETTINGS
+		GeneralSettings,
+		PathSettings,
+		PerformanceSettings,
+		AudioSettings,
+		MidiSettings
 	} ;
 
-	setupDialog( configTabs _tab_to_open = GENERAL_SETTINGS );
+	setupDialog( ConfigTabs _tab_to_open = GeneralSettings );
 	virtual ~setupDialog();
 
 
@@ -77,13 +69,15 @@ private slots:
 	void resetBufSize( void );
 	void displayBufSizeHelp( void );
 
-	// directory settings widget
+	// path settings widget
 	void setWorkingDir( const QString & _wd );
 	void setVSTDir( const QString & _vd );
 	void setArtworkDir( const QString & _ad );
 	void setFLDir( const QString & _fd );
-	void setLADSPADir( const QString & _fd );
-	void setSTKDir( const QString & _fd );
+	void setLADSPADir( const QString & _ld );
+	void setSTKDir( const QString & _sd );
+	void setDefaultSoundfont( const QString & _sf );
+	void setBackgroundArtwork( const QString & _ba );
 	
 	// audio settings widget
 	void audioInterfaceChanged( const QString & _driver );
@@ -94,13 +88,11 @@ private slots:
 	void displayMIDIHelp( void );
 
 
-	void toggleToolTips( bool _disabled );
-	void toggleKnobUsability( bool _classical );
-	void toggleGIMPLikeWindows( bool _enabled );
-	void toggleNoWizard( bool _enabled );
-	void toggleNoMsgAfterSetup( bool _enabled );
+	void toggleToolTips( bool _enabled );
+	void toggleWarnAfterSetup( bool _enabled );
 	void toggleDisplaydBV( bool _enabled );
-	void toggleNoMMPZ( bool _enabled );
+	void toggleMMPZ( bool _enabled );
+	void toggleHQAudioDev( bool _enabled );
 
 	void openWorkingDir( void );
 	void openVSTDir( void );
@@ -108,10 +100,11 @@ private slots:
 	void openFLDir( void );
 	void openLADSPADir( void );
 	void openSTKDir( void );
+	void openDefaultSoundfont( void );
+	void openBackgroundArtwork( void );
 
 	void toggleDisableChActInd( bool _disabled );
 	void toggleManualChPiano( bool _enabled );
-	void setParallelizingLevel( int _level );
 
 
 private:
@@ -121,13 +114,11 @@ private:
 	QLabel * m_bufSizeLbl;
 	int m_bufferSize;
 
-	bool m_disableToolTips;
-	bool m_classicalKnobUsability;
-	bool m_gimpLikeWindows;
-	bool m_noWizard;
-	bool m_noMsgAfterSetup;
+	bool m_toolTips;
+	bool m_warnAfterSetup;
 	bool m_displaydBV;
-	bool m_noMMPZ;
+	bool m_MMPZ;
+	bool m_hqAudioDev;
 
 
 	QLineEdit * m_wdLineEdit;
@@ -135,22 +126,29 @@ private:
 	QLineEdit * m_adLineEdit;
 	QLineEdit * m_fdLineEdit;
 	QLineEdit * m_ladLineEdit;
-#ifdef HAVE_STK_H
+#ifdef LMMS_HAVE_FLUIDSYNTH
+	QLineEdit * m_sfLineEdit;
+#endif
+#ifdef LMMS_HAVE_STK
 	QLineEdit * m_stkLineEdit;
 #endif
+	QLineEdit * m_baLineEdit;
 
 	QString m_workingDir;
 	QString m_vstDir;
 	QString m_artworkDir;
 	QString m_flDir;
 	QString m_ladDir;
-#ifdef HAVE_STK_H
+#ifdef LMMS_HAVE_FLUIDSYNTH
+	QString m_defaultSoundfont;
+#endif
+#ifdef LMMS_HAVE_STK
 	QString m_stkDir;
 #endif
+	QString m_backgroundArtwork;
 
 	bool m_disableChActInd;
 	bool m_manualChPiano;
-	int m_parLevel;
 
 	typedef QMap<QString, audioDevice::setupWidget *> aswMap;
 	typedef QMap<QString, midiClient::setupWidget *> mswMap;
