@@ -1,7 +1,7 @@
 /*
  * side_bar.h - code for side-bar in LMMS
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,20 +26,10 @@
 #ifndef _SIDE_BAR_H
 #define _SIDE_BAR_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QMap>
 
-#else
-
-#include <qmap.h>
-
-#endif
-
-
 #include "kmultitabbar.h"
+#include "side_bar_widget.h"
 
 
 class sideBar : public KMultiTabBar
