@@ -1,7 +1,7 @@
 /*
  * tab_bar.h - class tabBar
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -26,32 +26,17 @@
 #ifndef _TAB_BAR_H
 #define _TAB_BAR_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include "qt3support.h"
-
-#ifdef QT4
-
-#include <QWidget>
-#include <QMap>
-#include <QLayout>
-
-#else
-
-#include <qwidget.h>
-#include <qmap.h>
-#include <qlayout.h>
-
-#endif
+#include <QtCore/QMap>
+#include <QtGui/QLayout>
+#include <QtGui/QWidget>
 
+#include "export.h"
 
 
 class tabButton;
 
 
-class tabBar : public QWidget
+class EXPORT tabBar : public QWidget
 {
 	Q_OBJECT
 public:
@@ -59,10 +44,10 @@ public:
 			QBoxLayout::Direction _dir = QBoxLayout::LeftToRight );
 	virtual ~tabBar();
 
-	tabButton * FASTCALL addTab( QWidget * _w, const QString & _text,
+	tabButton * addTab( QWidget * _w, const QString & _text,
 					int _id, bool _add_stretch = FALSE,
 					bool _text_is_tooltip = FALSE );
-	void FASTCALL removeTab( int _id );
+	void removeTab( int _id );
 
 	inline void setExclusive( bool _on )
 	{
@@ -77,8 +62,8 @@ public slots:
 
 
 protected:
-	bool FASTCALL tabState( int _id );
-	void FASTCALL setTabState( int _id, bool _checked );
+	bool tabState( int _id );
+	void setTabState( int _id, bool _checked );
 	bool allHidden( void );
 
 
