@@ -1,7 +1,7 @@
 /*
  * tab_button.h - declaration of class tabButton
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,19 +26,8 @@
 #ifndef _TAB_BUTTON_H
 #define _TAB_BUTTON_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QPushButton>
 
-#else
-
-#include <qpushbutton.h>
-
-#endif
-
-
 
 class tabButton : public QPushButton
 {
@@ -48,11 +37,7 @@ public:
 		QPushButton( _text, _parent ),
 		m_id( _id )
 	{
-#ifndef QT3
 		setCheckable( TRUE );
-#else
-		setToggleButton( TRUE );
-#endif
 		connect( this, SIGNAL( clicked() ), this,
 						SLOT( slotClicked() ) );
 	}
@@ -61,17 +46,6 @@ public:
 	{
 	}
 
-#ifdef QT3
-	inline void setChecked( bool _on )
-	{
-		setOn( _on );
-	}
-
-	inline bool isChecked( void ) const
-	{
-		return( isOn() );
-	}
-#endif
 
 signals:
 	void clicked( int );
