@@ -2,7 +2,7 @@
  * audio_file_processor.h - declaration of class audioFileProcessor
  *                          (instrument-plugin for using audio-files)
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,84 +27,84 @@
 #ifndef _AUDIO_FILE_PROCESSOR_H
 #define _AUDIO_FILE_PROCESSOR_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QPixmap>
 
-#else
-
-#include <qpixmap.h>
-
-#endif
-
 #include "instrument.h"
+#include "instrument_view.h"
 #include "sample_buffer.h"
-#include "spc_bg_hndl_widget.h"
+#include "knob.h"
+#include "pixmap_button.h"
 
 
-class knob;
-class pixmapButton;
-class volumeKnob;
 
-
-class audioFileProcessor : public instrument, public specialBgHandlingWidget
+class audioFileProcessor : public instrument
 {
 	Q_OBJECT
 public:
-	class subPluginFeatures : public plugin::descriptor::subPluginFeatures
-	{
-	public:
-		subPluginFeatures( plugin::pluginTypes _type );
-
-		virtual const QStringList & supportedExtensions( void )
-		{
-			return( supported_extensions() );
-		}
-
-		static const QStringList & supported_extensions( void );
-
-	} ;
-
-
-	audioFileProcessor( instrumentTrack * _channel_track );
+	audioFileProcessor( instrumentTrack * _instrument_track );
 	virtual ~audioFileProcessor();
 
-	virtual void FASTCALL playNote( notePlayHandle * _n,
-						bool _try_parallelizing );
-	virtual void FASTCALL deleteNotePluginData( notePlayHandle * _n );
+	virtual void playNote( notePlayHandle * _n, 
+						sampleFrame * _working_buffer );
+	virtual void deleteNotePluginData( notePlayHandle * _n );
 
-	virtual void FASTCALL saveSettings( QDomDocument & _doc,
+	virtual void saveSettings( QDomDocument & _doc,
 						QDomElement & _parent );
-	virtual void FASTCALL loadSettings( const QDomElement & _this );
+	virtual void loadSettings( const QDomElement & _this );
 
-	virtual void FASTCALL setParameter( const QString & _param,
-						const QString & _value );
+	virtual void loadFile( const QString & _file );
 
 	virtual QString nodeName( void ) const;
 
-	virtual Uint32 FASTCALL getBeatLen( notePlayHandle * _n ) const;
+	virtual Uint32 getBeatLen( notePlayHandle * _n ) const;
 
 	virtual f_cnt_t desiredReleaseFrames( void ) const
 	{
 		return( 128 );
 	}
 
+	virtual pluginView * instantiateView( QWidget * _parent );
+
 
 public slots:
 	void setAudioFile( const QString & _audio_file, bool _rename = TRUE );
 
 
+private slots:
+	void reverseModelChanged( void );
+	void ampModelChanged( void );
+	void loopPointChanged( void );
+
+
+private:
+	typedef sampleBuffer::handleState handleState;
+
+	sampleBuffer m_sampleBuffer;
+	
+	knobModel m_ampModel;
+	knobModel m_startPointModel;
+	knobModel m_endPointModel;
+	boolModel m_reverseModel;
+	boolModel m_loopModel;
+
+
+	friend class audioFileProcessorView;
+
+} ;
+
+
+
+class audioFileProcessorView : public instrumentView
+{
+	Q_OBJECT
+public:
+	audioFileProcessorView( instrument * _instrument, QWidget * _parent );
+	virtual ~audioFileProcessorView();
+
+
 protected slots:
-	void openAudioFile( void );
-	void reverseBtnToggled( bool _on );
-	void ampKnobChanged( float _new_value );
-	void startKnobChanged( float _new_value );
-	void endKnobChanged( float _new_value );
-	void lineDrawBtnToggled( bool _on );
-	void dotDrawBtnToggled( bool _on );
 	void sampleUpdated( void );
+	void openAudioFile( void );
 
 
 protected:
@@ -114,29 +114,21 @@ protected:
 
 
 private:
-	typedef sampleBuffer::handleState handleState;
+	virtual void modelChanged( void );
 
 	static QPixmap * s_artwork;
 
-
-	sampleBuffer m_sampleBuffer;
-	
-	sampleBuffer::drawMethods m_drawMethod;
-
 	QPixmap m_graph;
-	volumeKnob * m_ampKnob;
+	knob * m_ampKnob;
 	knob * m_startKnob;
 	knob * m_endKnob;
 	pixmapButton * m_openAudioFileButton;
-	pixmapButton * m_viewLinesPB;
-	pixmapButton * m_viewDotsPB;
 	pixmapButton * m_reverseButton;
 	pixmapButton * m_loopButton;
 
-
-	void updateSample( void );
-
 } ;
 
 
+
+
 #endif
