@@ -1,7 +1,7 @@
 /*
  * ladspa_effect.cpp - class for processing LADSPA effects
  *
- * Copyright (c) 2006-2007 Danny McRae <khjklujn/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Danny McRae <khjklujn/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -23,27 +23,18 @@
  */
 
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtGui/QMessageBox>
 
-#else
-
-#include "qmessagebox.h"
-#define indexOf find
-
-#endif
-
-#include <malloc.h>
-
 #include "ladspa_effect.h"
-#include "mixer.h"
-#include "config_mgr.h"
+#include "mmp.h"
 #include "audio_device.h"
+#include "config_mgr.h"
+#include "ladspa_2_lmms.h"
 #include "ladspa_control.h"
 #include "ladspa_subplugin_features.h"
+#include "mixer.h"
+#include "effect_chain.h"
+#include "automation_pattern.h"
 
 
 #undef SINGLE_SOURCE_COMPILE
@@ -53,9 +44,9 @@
 extern "C"
 {
 
-plugin::descriptor ladspaeffect_plugin_descriptor =
+plugin::descriptor PLUGIN_EXPORT ladspaeffect_plugin_descriptor =
 {
-	STRINGIFY_PLUGIN_NAME( PLUGIN_NAME ),
+	STRINGIFY( PLUGIN_NAME ),
 	"LADSPA Effect",
 	QT_TRANSLATE_NOOP( "pluginBrowser",
 				"plugin for using arbitrary LADSPA-effects "
@@ -63,54 +54,263 @@ plugin::descriptor ladspaeffect_plugin_descriptor =
 	"Danny McRae <khjklujn/at/users.sourceforge.net>",
 	0x0100,
 	plugin::Effect,
-	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) ),
+	new pluginPixmapLoader( "logo" ),
+	NULL,
 	new ladspaSubPluginFeatures( plugin::Effect )
 } ;
 
 }
 
 
-ladspaEffect::ladspaEffect( const descriptor::subPluginFeatures::key * _key ) :
-	effect( &ladspaeffect_plugin_descriptor, _key ),
-	m_effName( "none" ),
-	m_key( subPluginKeyToLadspaKey( _key )
-		/* ladspa_key_t( _cdata->settings.attribute( "label" ),
-				_cdata->settings.attribute( "lib" ) )*/ ),
-	m_ladspa( engine::getLADSPAManager() )
+ladspaEffect::ladspaEffect( model * _parent,
+			const descriptor::subPluginFeatures::key * _key ) :
+	effect( &ladspaeffect_plugin_descriptor, _parent, _key ),
+	m_controls( NULL ),
+	m_maxSampleRate( 0 ),
+	m_key( ladspaSubPluginFeatures::subPluginKeyToLadspaKey( _key ) )
 {
-	if( m_ladspa->getDescription( m_key ) == NULL )
+	ladspa2LMMS * manager = engine::getLADSPAManager();
+	if( manager->getDescription( m_key ) == NULL )
 	{
-		QMessageBox::warning( 0, "Effect", 
-			"Unknown LADSPA plugin requested: " + m_key.first, 
-			QMessageBox::Ok, QMessageBox::NoButton );
+		if( !engine::suppressMessages() )
+		{
+			QMessageBox::warning( 0, tr( "Effect" ), 
+				tr( "Unknown LADSPA plugin %1 requested." ).
+							arg( m_key.second ),
+				QMessageBox::Ok, QMessageBox::NoButton );
+		}
 		setOkay( FALSE );
 		return;
 	}
-	
-	setPublicName( m_ladspa->getShortName( m_key ) );
-	
+
+	setDisplayName( manager->getShortName( m_key ) );
+
+	pluginInstantiation();
+
+	connect( engine::getMixer(), SIGNAL( sampleRateChanged() ),
+					this, SLOT( changeSampleRate() ) );
+}
+
+
+
+
+ladspaEffect::~ladspaEffect()
+{
+	pluginDestruction();
+}
+
+
+
+
+void ladspaEffect::changeSampleRate( void )
+{
+	multimediaProject mmp( multimediaProject::EffectSettings );
+	m_controls->saveState( mmp, mmp.content() );
+
+	ladspaControls * controls = m_controls;
+	m_controls = NULL;
+
+	m_pluginMutex.lock();
+	pluginDestruction();
+	pluginInstantiation();
+	m_pluginMutex.unlock();
+
+	controls->effectModelChanged( m_controls );
+	delete controls;
+
+	m_controls->restoreState( mmp.content().firstChild().toElement() );
+
+	// the IDs of re-created controls have been saved and now need to be
+	// resolved again
+	automationPattern::resolveAllIDs();
+}
+
+
+
+
+bool ladspaEffect::processAudioBuffer( sampleFrame * _buf, 
+							const fpp_t _frames )
+{
+	m_pluginMutex.lock();
+	if( !isOkay() || dontRun() || !isRunning() || !isEnabled() )
+	{
+		m_pluginMutex.unlock();
+		return( FALSE );
+	}
+
+	int frames = _frames;
+	sampleFrame * o_buf = NULL;
+
+	if( m_maxSampleRate < engine::getMixer()->processingSampleRate() )
+	{
+		o_buf = _buf;
+		_buf = new sampleFrame[_frames];
+		sampleDown( o_buf, _buf, m_maxSampleRate );
+		frames = _frames * m_maxSampleRate /
+				engine::getMixer()->processingSampleRate();
+	}
+
+	// Copy the LMMS audio buffer to the LADSPA input buffer and initialize
+	// the control ports.  Need to change this to handle non-in-place-broken
+	// plugins--would speed things up to use the same buffer for both
+	// LMMS and LADSPA.
+	ch_cnt_t channel = 0;
+	for( ch_cnt_t proc = 0; proc < getProcessorCount(); ++proc )
+	{
+		for( int port = 0; port < m_portCount; ++port )
+		{
+			port_desc_t * pp = m_ports.at( proc ).at( port );
+			switch( pp->rate )
+			{
+				case CHANNEL_IN:
+					for( fpp_t frame = 0; 
+						frame < frames; ++frame )
+					{
+						pp->buffer[frame] = 
+							_buf[frame][channel];
+					}
+					++channel;
+					break;
+				case AUDIO_RATE_INPUT:
+					pp->value = 
+						static_cast<LADSPA_Data>( 
+							pp->control->getValue() /
+								pp->scale );
+					// This only supports control rate ports, so the audio rates are
+					// treated as though they were control rate by setting the
+					// port buffer to all the same value.
+					for( fpp_t frame = 0; 
+						frame < frames; ++frame )
+					{
+						pp->buffer[frame] = 
+							pp->value;
+					}
+					break;
+				case CONTROL_RATE_INPUT:
+					if( pp->control == NULL )
+					{
+						break;
+					}
+					pp->value = 
+						static_cast<LADSPA_Data>( 
+							pp->control->getValue() /
+								pp->scale );
+					pp->buffer[0] = 
+						pp->value;
+					break;
+				case CHANNEL_OUT:
+				case AUDIO_RATE_OUTPUT:
+				case CONTROL_RATE_OUTPUT:
+					break;
+				default:
+					break;
+			}
+		}
+	}
+
+	// Process the buffers.
+	for( ch_cnt_t proc = 0; proc < getProcessorCount(); ++proc )
+	{
+		(m_descriptor->run)( m_handles[proc], frames );
+	}
+
+	// Copy the LADSPA output buffers to the LMMS buffer.
+	double out_sum = 0.0;
+	channel = 0;
+	const float d = getDryLevel();
+	const float w = getWetLevel();
+	for( ch_cnt_t proc = 0; proc < getProcessorCount(); ++proc )
+	{
+		for( int port = 0; port < m_portCount; ++port )
+		{
+			port_desc_t * pp = m_ports.at( proc ).at( port );
+			switch( pp->rate )
+			{
+				case CHANNEL_IN:
+				case AUDIO_RATE_INPUT:
+				case CONTROL_RATE_INPUT:
+					break;
+				case CHANNEL_OUT:
+					for( fpp_t frame = 0; 
+						frame < frames; ++frame )
+					{
+						_buf[frame][channel] = 
+							d * 
+							_buf[frame][channel] +
+							w *
+							pp->buffer[frame];
+						out_sum += 
+							_buf[frame][channel] *
+							_buf[frame][channel];
+					}
+					++channel;
+					break;
+				case AUDIO_RATE_OUTPUT:
+				case CONTROL_RATE_OUTPUT:
+					break;
+				default:
+					break;
+			}
+		}
+	}
+
+	if( o_buf != NULL )
+	{
+		sampleBack( _buf, o_buf, m_maxSampleRate );
+		delete[] _buf;
+	}
+
+	checkGate( out_sum / frames );
+
+
+	bool is_running = isRunning();
+	m_pluginMutex.unlock();
+	return( is_running );
+}
+
+
+
+
+void ladspaEffect::setControl( int _control, LADSPA_Data _value )
+{
+	if( !isOkay() )
+	{
+		return;
+	}
+	m_portControls[_control]->value = _value;
+}
+
+
+
+
+void ladspaEffect::pluginInstantiation( void )
+{
+	m_maxSampleRate = maxSamplerate( displayName() );
+
+	ladspa2LMMS * manager = engine::getLADSPAManager();
+
 	// Calculate how many processing units are needed.
 	const ch_cnt_t lmms_chnls = engine::getMixer()->audioDev()->channels();
-	m_effectChannels = m_ladspa->getDescription( m_key )->inputChannels;
-	setProcessorCount( lmms_chnls / m_effectChannels );
-	
+	int effect_channels = manager->getDescription( m_key )->inputChannels;
+	setProcessorCount( lmms_chnls / effect_channels );
+
 	// Categorize the ports, and create the buffers.
-	m_portCount = m_ladspa->getPortCount( m_key );
-	
+	m_portCount = manager->getPortCount( m_key );
+
 	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++ )
 	{
 		multi_proc_t ports;
-		for( Uint16 port = 0; port < m_portCount; port++ )
+		for( int port = 0; port < m_portCount; port++ )
 		{
 			port_desc_t * p = new portDescription;
-			
-			p->name = m_ladspa->getPortName( m_key, port );
+
+			p->name = manager->getPortName( m_key, port );
 			p->proc = proc;
 			p->port_id = port;
 			p->control = NULL;
-			
+
 			// Determine the port's category.
-			if( m_ladspa->isPortAudio( m_key, port ) )
+			if( manager->isPortAudio( m_key, port ) )
 			{
 		// Nasty manual memory management--was having difficulty
 		// with some prepackaged plugins that were segfaulting
@@ -118,18 +318,18 @@ ladspaEffect::ladspaEffect( const descriptor::subPluginFeatures::key * _key ) :
 		// memory management all taking place in one file.
 				p->buffer = 
 		new LADSPA_Data[engine::getMixer()->framesPerPeriod()];
-				
-				if( p->name.toUpper().contains( "IN" ) && 
-					m_ladspa->isPortInput( m_key, port ) )
+
+				if( p->name.toUpper().contains( "IN" ) &&
+					manager->isPortInput( m_key, port ) )
 				{
 					p->rate = CHANNEL_IN;
 				}
-				else if( p->name.toUpper().contains( "OUT" ) && 
-					m_ladspa->isPortOutput( m_key, port ) )
+				else if( p->name.toUpper().contains( "OUT" ) &&
+					manager->isPortOutput( m_key, port ) )
 				{
 					p->rate = CHANNEL_OUT;
 				}
-				else if( m_ladspa->isPortInput( m_key, port ) )
+				else if( manager->isPortInput( m_key, port ) )
 				{
 					p->rate = AUDIO_RATE_INPUT;
 				}
@@ -140,9 +340,9 @@ ladspaEffect::ladspaEffect( const descriptor::subPluginFeatures::key * _key ) :
 			}
 			else
 			{
-				p->buffer = new LADSPA_Data;
-				
-				if( m_ladspa->isPortInput( m_key, port ) )
+				p->buffer = new LADSPA_Data[1];
+
+				if( manager->isPortInput( m_key, port ) )
 				{
 					p->rate = CONTROL_RATE_INPUT;
 				}
@@ -151,13 +351,13 @@ ladspaEffect::ladspaEffect( const descriptor::subPluginFeatures::key * _key ) :
 					p->rate = CONTROL_RATE_OUTPUT;
 				}
 			}
-			
+
 			p->scale = 1.0f;
-			if( m_ladspa->isPortToggled( m_key, port ) )
+			if( manager->isPortToggled( m_key, port ) )
 			{
 				p->data_type = TOGGLED;
 			}
-			else if( m_ladspa->isInteger( m_key, port ) )
+			else if( manager->isInteger( m_key, port ) )
 			{
 				p->data_type = INTEGER;
 			}
@@ -184,36 +384,36 @@ ladspaEffect::ladspaEffect( const descriptor::subPluginFeatures::key * _key ) :
 			}
 			else
 			{
-				p->data_type = FLOAT;
+				p->data_type = FLOATING;
 			}
-			
+
 			// Get the range and default values.
-			p->max = m_ladspa->getUpperBound( m_key, port );
+			p->max = manager->getUpperBound( m_key, port );
 			if( p->max == NOHINT )
 			{
 				p->max = p->name.toUpper() == "GAIN" ? 10.0f :
 					1.0f;
 			}
-			
-			if( m_ladspa->areHintsSampleRateDependent( 
+
+			if( manager->areHintsSampleRateDependent(
 								m_key, port ) )
 			{
-				p->max *= engine::getMixer()->sampleRate();
+				p->max *= m_maxSampleRate;
 			}
-			
-			p->min = m_ladspa->getLowerBound( m_key, port );
+
+			p->min = manager->getLowerBound( m_key, port );
 			if( p->min == NOHINT )
 			{
 				p->min = 0.0f;
 			}
-			
-			if( m_ladspa->areHintsSampleRateDependent( 
+
+			if( manager->areHintsSampleRateDependent(
 								m_key, port ) )
 			{
-				p->min *= engine::getMixer()->sampleRate();
+				p->min *= m_maxSampleRate;
 			}
-				
-			p->def = m_ladspa->getDefaultSetting( m_key, port );
+
+			p->def = manager->getDefaultSetting( m_key, port );
 			if( p->def == NOHINT )
 			{
 				if( p->data_type != TOGGLED )
@@ -225,268 +425,158 @@ ladspaEffect::ladspaEffect( const descriptor::subPluginFeatures::key * _key ) :
 					p->def = 1.0f;
 				}
 			}
-			
+
 			p->max *= p->scale;
 			p->min *= p->scale;
 			p->def *= p->scale;
-			
+
 			p->value = p->def;
-			
-			
+
+
 			ports.append( p );
-			
+
 	// For convenience, keep a separate list of the ports that are used 
 	// to control the processors.
 			if( p->rate == AUDIO_RATE_INPUT || 
 					p->rate == CONTROL_RATE_INPUT )
 			{
-				p->control_id = m_controls.count();
-				m_controls.append( p );
+				p->control_id = m_portControls.count();
+				m_portControls.append( p );
 			}
 		}
 		m_ports.append( ports );
 	}
-	
+
 	// Instantiate the processing units.
-	m_descriptor = m_ladspa->getDescriptor( m_key );
+	m_descriptor = manager->getDescriptor( m_key );
 	if( m_descriptor == NULL )
 	{
 		QMessageBox::warning( 0, "Effect", 
-			"Can't get LADSPA descriptor function: " + m_key.first,
+			"Can't get LADSPA descriptor function: " + m_key.second,
 			QMessageBox::Ok, QMessageBox::NoButton );
 		setOkay( FALSE );
 		return;
 	}
 	if( m_descriptor->run == NULL )
 	{
-		QMessageBox::warning( 0, "Effect", 
-			"Plugin has no processor: " + m_key.first,
+		QMessageBox::warning( 0, "Effect",
+			"Plugin has no processor: " + m_key.second,
 			QMessageBox::Ok, QMessageBox::NoButton );
 		setDontRun( TRUE );
 	}
 	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++ )
 	{
-		LADSPA_Handle effect = m_ladspa->instantiate( m_key,
-					engine::getMixer()->sampleRate() );
+		LADSPA_Handle effect = manager->instantiate( m_key,
+							m_maxSampleRate );
 		if( effect == NULL )
 		{
-			QMessageBox::warning( 0, "Effect", 
-				"Can't get LADSPA instance: " + m_key.first,
+			QMessageBox::warning( 0, "Effect",
+				"Can't get LADSPA instance: " + m_key.second,
 				QMessageBox::Ok, QMessageBox::NoButton );
 			setOkay( FALSE );
 			return;
 		}
 		m_handles.append( effect );
-		
 	}
-	
+
 	// Connect the ports.
 	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++ )
 	{
-		for( Uint16 port = 0; port < m_portCount; port++ )
+		for( int port = 0; port < m_portCount; port++ )
 		{
-			if( !m_ladspa->connectPort( m_key, 
-			     			m_handles[proc], 
-						port, 
-						m_ports[proc][port]->buffer ) )
+			port_desc_t * pp = m_ports.at( proc ).at( port );
+			if( !manager->connectPort( m_key,
+			     			m_handles[proc],
+						port,
+						pp->buffer ) )
 			{
 				QMessageBox::warning( 0, "Effect", 
-				"Failed to connect port: " + m_key.first, 
+				"Failed to connect port: " + m_key.second, 
 				QMessageBox::Ok, QMessageBox::NoButton );
 				setDontRun( TRUE );
 				return;
 			}
 		}
 	}
-	
+
 	// Activate the processing units.
 	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++ )
 	{
-		m_ladspa->activate( m_key, m_handles[proc] );
+		manager->activate( m_key, m_handles[proc] );
 	}
+	m_controls = new ladspaControls( this );
 }
 
 
 
 
-ladspaEffect::~ladspaEffect()
+void ladspaEffect::pluginDestruction( void )
 {
 	if( !isOkay() )
 	{
 		return;
 	}
-	
+
+	delete m_controls;
+
 	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++ )
 	{
-		m_ladspa->deactivate( m_key, m_handles[proc] );
-		m_ladspa->cleanup( m_key, m_handles[proc] );
-		for( Uint16 port = 0; port < m_portCount; port++ )
+		ladspa2LMMS * manager = engine::getLADSPAManager();
+		manager->deactivate( m_key, m_handles[proc] );
+		manager->cleanup( m_key, m_handles[proc] );
+		for( int port = 0; port < m_portCount; port++ )
 		{
-			free( m_ports[proc][port]->buffer );
-			free( m_ports[proc][port] );
+			port_desc_t * pp = m_ports.at( proc ).at( port );
+			delete[] pp->buffer;
+			delete pp;
 		}
 		m_ports[proc].clear();
 	}
 	m_ports.clear();
 	m_handles.clear();
+	m_portControls.clear();
 }
 
 
 
 
-bool FASTCALL ladspaEffect::processAudioBuffer( surroundSampleFrame * _buf, 
-							const fpp_t _frames )
+
+
+static QMap<QString, sample_rate_t> __buggy_plugins;
+
+sample_rate_t ladspaEffect::maxSamplerate( const QString & _name )
 {
-	if( !isOkay() || dontRun() || !isRunning() || isBypassed() )
+	if( __buggy_plugins.isEmpty() )
 	{
-		return( FALSE );
+		__buggy_plugins["C* AmpVTS"] = 88200;
+		__buggy_plugins["Chorus2"] = 44100;
+		__buggy_plugins["Notch Filter"] = 96000;
+		__buggy_plugins["Freeverb"] = 44100;
+		__buggy_plugins["TAP Reflector"] = 192000;
 	}
-	
-	// Copy the LMMS audio buffer to the LADSPA input buffer and initialize
-	// the control ports.  Need to change this to handle non-in-place-broken
-	// plugins--would speed things up to use the same buffer for both
-	// LMMS and LADSPA.
-	ch_cnt_t channel = 0;
-	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++)
+	if( __buggy_plugins.contains( _name ) )
 	{
-		for( Uint16 port = 0; port < m_portCount; port++ )
-		{
-			switch( m_ports[proc][port]->rate )
-			{
-				case CHANNEL_IN:
-					for( fpp_t frame = 0; 
-						frame < _frames; frame++ )
-					{
-						m_ports[proc][port]->buffer[frame] = 
-							_buf[frame][channel];
-					}
-					channel++;
-					break;
-				case AUDIO_RATE_INPUT:
-					m_ports[proc][port]->value = 
-						static_cast<LADSPA_Data>( 
-							m_ports[proc][port]->control->getValue() /
-								m_ports[proc][port]->scale );
-					// This only supports control rate ports, so the audio rates are
-					// treated as though they were control rate by setting the
-					// port buffer to all the same value.
-					for( fpp_t frame = 0; 
-						frame < _frames; frame++ )
-					{
-						m_ports[proc][port]->buffer[frame] = 
-							m_ports[proc][port]->value;
-					}
-					break;
-				case CONTROL_RATE_INPUT:
-					if( m_ports[proc][port]->control ==
-									NULL )
-					{
-						break;
-					}
-					m_ports[proc][port]->value = 
-						static_cast<LADSPA_Data>( 
-							m_ports[proc][port]->control->getValue() /
-								m_ports[proc][port]->scale );
-					m_ports[proc][port]->buffer[0] = 
-						m_ports[proc][port]->value;
-					break;
-				case CHANNEL_OUT:
-				case AUDIO_RATE_OUTPUT:
-				case CONTROL_RATE_OUTPUT:
-					break;
-				default:
-					break;
-			}
-		}
+		return( __buggy_plugins[_name] );
 	}
-	
-	// Process the buffers.
-	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++ )
-	{
-		(m_descriptor->run)(m_handles[proc], _frames);
-	}
-	
-	// Copy the LADSPA output buffers to the LMMS buffer.
-	double out_sum = 0.0;
-	channel = 0;
-	for( ch_cnt_t proc = 0; proc < getProcessorCount(); proc++)
-	{
-		for( Uint16 port = 0; port < m_portCount; port++ )
-		{
-			switch( m_ports[proc][port]->rate )
-			{
-				case CHANNEL_IN:
-				case AUDIO_RATE_INPUT:
-				case CONTROL_RATE_INPUT:
-					break;
-				case CHANNEL_OUT:
-					for( fpp_t frame = 0; 
-						frame < _frames; frame++ )
-					{
-						_buf[frame][channel] = 
-							getDryLevel() * 
-							_buf[frame][channel] +
-							getWetLevel() *
-							m_ports[proc][port]->buffer[frame];
-						out_sum += 
-							_buf[frame][channel] *
-							_buf[frame][channel];
-					}
-					channel++;
-					break;
-				case AUDIO_RATE_OUTPUT:
-				case CONTROL_RATE_OUTPUT:
-					break;
-				default:
-					break;
-			}
-		}
-	}
-	
-	// Check whether we need to continue processing input.  Restart the
-	// counter if the threshold has been exceeded.
-	if( out_sum <= getGate() )
-	{
-		incrementBufferCount();
-		if( getBufferCount() > getTimeout() )
-		{
-			stopRunning();
-			resetBufferCount();
-		}
-	}
-	else
-	{
-		resetBufferCount();
-	}
-	
-	return( isRunning() );
+	return( engine::getMixer()->processingSampleRate() );
 }
 
 
 
 
-void FASTCALL ladspaEffect::setControl( Uint16 _control, LADSPA_Data _value )
-{
-	if( !isOkay() )
-	{
-		return;
-	}
-	m_controls[_control]->value = _value;
-}
-
-
-#undef indexOf
-
 extern "C"
 {
 
 // neccessary for getting instance out of shared lib
-plugin * lmms_plugin_main( void * _data )
+plugin * PLUGIN_EXPORT lmms_plugin_main( model * _parent, void * _data )
 {
-	return( new ladspaEffect(
+	return( new ladspaEffect( _parent,
 		static_cast<const plugin::descriptor::subPluginFeatures::key *>(
 								_data ) ) );
 }
 
 }
 
+
+#include "moc_ladspa_effect.cxx"
+
