@@ -1,7 +1,7 @@
 /*
  * midi_import.h - support for importing MIDI-files
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,28 +26,10 @@
 #ifndef _MIDI_IMPORT_H
 #define _MIDI_IMPORT_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QString>
 #include <QtCore/QPair>
 #include <QtCore/QVector>
 
-#else
-
-#include <qstring.h>
-#include <qpair.h>
-#include <qvaluevector.h>
-
-#endif
-
-
 #include "midi.h"
 #include "import_filter.h"
 
@@ -58,13 +40,18 @@ public:
 	midiImport( const QString & _file );
 	virtual ~midiImport();
 
+	virtual pluginView * instantiateView( QWidget * )
+	{
+		return( NULL );
+	}
+
 
 private:
 	virtual bool tryImport( trackContainer * _tc );
 
-	bool FASTCALL readSMF( trackContainer * _tc );
-	bool FASTCALL readRIFF( trackContainer * _tc );
-	bool FASTCALL readTrack( int _track_end, QString & _track_name );
+	bool readSMF( trackContainer * _tc );
+	bool readRIFF( trackContainer * _tc );
+	bool readTrack( int _track_end, QString & _track_name );
 
 	void error( void );
 
@@ -131,7 +118,7 @@ private:
 	}
 
 
-	typedef vvector<QPair<int, midiEvent> > eventVector;
+	typedef QVector<QPair<int, midiEvent> > eventVector;
 	eventVector m_events;
 	int m_timingDivision;
 
