@@ -24,18 +24,17 @@
 #ifndef _NINE_BUTTON_SELECTOR_H
 #define _NINE_BUTTON_SELECTOR_H
 
-#include "qt3support.h"
-#include "config.h"
-#include "types.h"
 #include "pixmap_button.h"
 
 
-class nineButtonSelector: public QWidget
+class nineButtonSelector: public QWidget , public intModelView
 {
 	Q_OBJECT
 			
 public:
-	nineButtonSelector( 	QPixmap _button1_on,
+	nineButtonSelector( 	QPixmap _button0_on,
+				QPixmap _button0_off,
+				QPixmap _button1_on,
 				QPixmap _button1_off,
 				QPixmap _button2_on,
 				QPixmap _button2_off,
@@ -51,16 +50,17 @@ public:
 				QPixmap _button7_off,
 				QPixmap _button8_on,
 				QPixmap _button8_off,
-				QPixmap _button9_on,
-				QPixmap _button9_off,
 				Uint8 _default,
 				Uint32 _x, Uint32 _y,
-				QWidget * _parent,
-				track * _track );
-	~nineButtonSelector();
+				QWidget * _parent);
+	virtual ~nineButtonSelector();
 	
-	inline Uint8 getSelected() { return( m_selected ); };
-	void FASTCALL setSelected( Uint8 _new_button );
+//	inline Uint8 getSelected() { 
+//		return( castModel<nineButtonSelectorModel>()->value() );
+//	};
+
+protected:
+	void setSelected( Uint8 _new_button );
 	
 public slots:
 	void button0Clicked( void );
@@ -79,11 +79,18 @@ signals:
 	void nineButtonSelection( Uint8 );
 	
 private:
-	vlist<pixmapButton *> m_buttons;
+	virtual void modelChanged( void );
+	void updateButton( Uint8 );
+
+	QList<pixmapButton *> m_buttons;
 	pixmapButton * m_button;
 	pixmapButton * m_lastBtn;
 	QPixmap m_base;
-	
+
 	Uint8 m_selected;
+	
 };
+
+typedef intModel nineButtonSelectorModel;
+
 #endif
