@@ -3,7 +3,7 @@
 /*
  * import_filter.cpp - base-class for all import-filters (MIDI, FLP etc)
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -25,27 +25,18 @@
  */
 
 
+#include <QtGui/QMessageBox>
+
 #include "import_filter.h"
 #include "engine.h"
 #include "track_container.h"
 #include "project_journal.h"
 
-#ifdef QT4
-
-#include <QtGui/QMessageBox>
-
-#else
-
-#include <qmessagebox.h>
-
-#define fileName name
-
-#endif
 
 
 importFilter::importFilter( const QString & _file_name,
 					const descriptor * _descriptor ) :
-	plugin( _descriptor ),
+	plugin( _descriptor, NULL ),
 	m_file( _file_name )
 {
 }
@@ -63,29 +54,23 @@ importFilter::~importFilter()
 void importFilter::import( const QString & _file_to_import,
 						trackContainer * _tc )
 {
-	vvector<descriptor> d;
+	QVector<descriptor> d;
 	plugin::getDescriptorsOfAvailPlugins( d );
 
 	bool successful = FALSE;
 
-	char * s = qstrdup( _file_to_import.
-#ifndef QT3
-			toAscii().constData()
-#else
-			ascii()
-#endif
-						);
+	char * s = qstrdup( _file_to_import.toAscii().constData() );
 
 	// do not record changes while importing files
 	const bool j = engine::getProjectJournal()->isJournalling();
 	engine::getProjectJournal()->setJournalling( FALSE );
 
-	for( vvector<plugin::descriptor>::iterator it = d.begin();
+	for( QVector<plugin::descriptor>::iterator it = d.begin();
 							it != d.end(); ++it )
 	{
 		if( it->type == plugin::ImportFilter )
 		{
-			plugin * p = plugin::instantiate( it->name, s );
+			plugin * p = plugin::instantiate( it->name, NULL, s );
 			if( dynamic_cast<importFilter *>( p ) != NULL &&
 				dynamic_cast<importFilter *>( p )->tryImport(
 								_tc ) == TRUE )
@@ -122,11 +107,7 @@ void importFilter::import( const QString & _file_to_import,
 
 bool importFilter::openFile( void )
 {
-#ifdef QT4
 	if( m_file.open( QFile::ReadOnly ) == FALSE )
-#else
-	if( m_file.open( IO_ReadOnly ) == FALSE )
-#endif
 	{
 		QMessageBox::critical( NULL,
 			trackContainer::tr( "Couldn't open file" ),
@@ -145,7 +126,6 @@ bool importFilter::openFile( void )
 }
 
 
-#undef fileName
 
 
 #endif
