@@ -2,9 +2,9 @@
 
 /*
  * note_play_handle.cpp - implementation of class notePlayHandle, part of
- *                        play-engine
+ *                        rendering engine
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -27,37 +27,25 @@
 
 
 #include "note_play_handle.h"
-#include "automatable_object_templates.h"
+#include "basic_filters.h"
 #include "config_mgr.h"
 #include "detuning_helper.h"
-#include "envelope_tab_widget.h"
+#include "instrument_sound_shaping.h"
 #include "instrument_track.h"
 #include "midi_port.h"
-#include "song_editor.h"
+#include "song.h"
 
 
 inline notePlayHandle::baseDetuning::baseDetuning(
 						detuningHelper * _detuning ) :
-	m_detuning( _detuning )
+	m_detuning( _detuning ),
+	m_value( m_detuning->getAutomationPattern()->valueAt( 0 ) )
 {
-	m_level = m_detuning->getAutomationPattern()->valueAt( 0 );
-	m_value = m_detuning->value( m_level );
 }
 
 
 
 
-inline void notePlayHandle::baseDetuning::setLevel( int _level )
-{
-	m_level = _level;
-	m_value = m_detuning->value( m_level );
-}
-
-
-
-
-
-
 
 
 notePlayHandle::notePlayHandle( instrumentTrack * _it,
@@ -65,9 +53,9 @@ notePlayHandle::notePlayHandle( instrumentTrack * _it,
 						const f_cnt_t _frames,
 						const note & _n,
 						notePlayHandle * _parent,
-						const bool _arp_note ) :
+						const bool _part_of_arp ) :
 	playHandle( NotePlayHandle, _offset ),
-	note( _n.length(), _n.pos(), _n.tone(), _n.octave(),
+	note( _n.length(), _n.pos(), _n.key(),
 			_n.getVolume(), _n.getPanning(), _n.detuning() ),
 	m_pluginData( NULL ),
 	m_filter( NULL ),
@@ -77,33 +65,34 @@ notePlayHandle::notePlayHandle( instrumentTrack * _it,
 	m_framesBeforeRelease( 0 ),
 	m_releaseFramesToDo( 0 ),
 	m_releaseFramesDone( 0 ),
-	m_released( FALSE ),
+	m_released( false ),
 	m_baseNote( _parent == NULL  ),
-	m_arpNote( _arp_note ),
-	m_muted( FALSE ),
+	m_partOfArpeggio( _part_of_arp ),
+	m_muted( false ),
 	m_bbTrack( NULL ),
-#if SINGERBOT_SUPPORT
+#ifdef LMMS_SINGERBOT_SUPPORT
 	m_patternIndex( 0 ),
 #endif
-	m_orig_bpm( engine::getSongEditor()->getTempo() )
+	m_origTempo( engine::getSong()->getTempo() )
 {
 	if( m_baseNote )
 	{
-		m_base_detuning = new baseDetuning( detuning() );
+		m_baseDetuning = new baseDetuning( detuning() );
 		m_instrumentTrack->m_processHandles.push_back( this );
 	}
 	else
 	{
-		m_base_detuning = _parent->m_base_detuning;
+		m_baseDetuning = _parent->m_baseDetuning;
 
 		_parent->m_subNotes.push_back( this );
 		// if there was an arp-note added and parent is a base-note
 		// we set arp-note-flag for indicating that parent is an
 		// arpeggio-base-note
-		_parent->m_arpNote = arpNote() && _parent->baseNote();
+		_parent->m_partOfArpeggio = isPartOfArpeggio() &&
+							_parent->isBaseNote();
 
 		m_bbTrack = _parent->m_bbTrack;
-#if SINGERBOT_SUPPORT
+#ifdef LMMS_SINGERBOT_SUPPORT
 		m_patternIndex = _parent->m_patternIndex;
 #endif
 	}
@@ -111,16 +100,17 @@ notePlayHandle::notePlayHandle( instrumentTrack * _it,
 	updateFrequency();
 
 	setFrames( _frames );
-	// send MIDI-note-on-event
-	m_instrumentTrack->processOutEvent( midiEvent( NOTE_ON,
-				m_instrumentTrack->m_midiPort->outputChannel(),
-					key(),
-				tLimit<Uint16>(
-				(Uint16) ( ( getVolume() / 100.0f ) *
-				( m_instrumentTrack->getVolume() / 100.0f ) *
-							127 ), 0, 127 ) ),
-			midiTime::fromFrames( offset(),
-						engine::framesPerTact64th() ) );
+
+
+	if( !isBaseNote() || !getInstrumentTrack()->arpeggiatorEnabled() )
+	{
+		// send MIDI-note-on-event
+		m_instrumentTrack->processOutEvent( midiEvent( MidiNoteOn,
+			m_instrumentTrack->getMidiPort()->outputChannel(),
+			key(), getMidiVelocity() ),
+				midiTime::fromFrames( offset(),
+						engine::framesPerTick() ) );
+	}
 }
 
 
@@ -128,17 +118,19 @@ notePlayHandle::notePlayHandle( instrumentTrack * _it,
 
 notePlayHandle::~notePlayHandle()
 {
-	if( m_released == FALSE )
-	{
-		noteOff( 0 );
-	}
+	noteOff( 0 );
 
 	if( m_baseNote )
 	{
-		delete m_base_detuning;
+		delete m_baseDetuning;
 		m_instrumentTrack->m_processHandles.removeAll( this );
 	}
 
+	if( m_instrumentTrack->m_notes[key()] == this )
+	{
+		m_instrumentTrack->m_notes[key()] = NULL;
+	}
+
 	if( m_pluginData != NULL )
 	{
 		m_instrumentTrack->deleteNotePluginData( this );
@@ -157,14 +149,39 @@ notePlayHandle::~notePlayHandle()
 
 
 
-void notePlayHandle::play( bool _try_parallelizing )
+void notePlayHandle::setVolume( const volume _volume )
 {
-	if( m_muted == TRUE )
+	note::setVolume( _volume );
+	m_instrumentTrack->processOutEvent( midiEvent( MidiKeyPressure,
+			m_instrumentTrack->getMidiPort()->outputChannel(),
+						key(), getMidiVelocity() ), 0 );
+	
+}
+
+
+
+
+int notePlayHandle::getMidiVelocity( void ) const
+{
+	int vel = getVolume();
+	if( m_instrumentTrack->getVolume() < DefaultVolume )
+	{
+		vel = ( vel * m_instrumentTrack->getVolume() ) / DefaultVolume;
+	}
+	return qMin( MidiMaxVelocity, vel * MidiMaxVelocity / DefaultVolume );
+}
+
+
+
+
+void notePlayHandle::play( sampleFrame * _working_buffer )
+{
+	if( m_muted )
 	{
 		return;
 	}
 
-	if( m_released == FALSE &&
+	if( m_released == false &&
 		m_totalFramesPlayed + engine::getMixer()->framesPerPeriod()
 								>= m_frames )
 	{
@@ -177,10 +194,10 @@ void notePlayHandle::play( bool _try_parallelizing )
 	if( framesLeft() > 0 )
 	{
 		// play note!
-		m_instrumentTrack->playNote( this, _try_parallelizing );
+		m_instrumentTrack->playNote( this, _working_buffer );
 	}
 
-	if( m_released == TRUE )
+	if( m_released )
 	{
 		f_cnt_t todo = engine::getMixer()->framesPerPeriod();
 		// if this note is base-note for arpeggio, always set
@@ -188,7 +205,7 @@ void notePlayHandle::play( bool _try_parallelizing )
 		// because we do not allow notePlayHandle::done() to be true
 		// until all sub-notes are completely played and no new ones
 		// are inserted by arpAndChordsTabWidget::processNote()
-		if( arpBaseNote() == TRUE )
+		if( isArpeggioBaseNote() )
 		{
 			m_releaseFramesToDo = m_releaseFramesDone + 2 *
 				engine::getMixer()->framesPerPeriod();
@@ -238,7 +255,7 @@ void notePlayHandle::play( bool _try_parallelizing )
 	for( notePlayHandleVector::iterator it = m_subNotes.begin();
 						it != m_subNotes.end(); )
 	{
-		( *it )->play( _try_parallelizing );
+		( *it )->play( _working_buffer );
 		if( ( *it )->done() )
 		{
 			delete *it;
@@ -254,7 +271,7 @@ void notePlayHandle::play( bool _try_parallelizing )
 	// can set m_releaseFramesDone to m_releaseFramesToDo so that
 	// notePlayHandle::done() returns true and also this base-note is
 	// removed from mixer's active note vector
-	if( arpBaseNote() == TRUE && m_subNotes.size() == 0 )
+	if( isArpeggioBaseNote() && m_subNotes.size() == 0 )
 	{
 		m_releaseFramesDone = m_releaseFramesToDo;
 	}
@@ -268,19 +285,16 @@ void notePlayHandle::play( bool _try_parallelizing )
 
 f_cnt_t notePlayHandle::framesLeft( void ) const
 {
-/*	const instrument * i = ( m_instrumentTrack != NULL ) ?
-				m_instrumentTrack->getInstrument() : NULL;
-	f_cnt_t rftd  = ( i != NULL && i->isMonophonic() ) ?
-						0 : actualReleaseFramesToDo();*/
 	if( m_released && actualReleaseFramesToDo() == 0 )
 	{
-		return( m_framesBeforeRelease );
+		return m_framesBeforeRelease;
 	}
 	else if( m_released && actualReleaseFramesToDo() >= m_releaseFramesDone )
 	{
-		return( m_framesBeforeRelease + actualReleaseFramesToDo() - m_releaseFramesDone );
+		return m_framesBeforeRelease + actualReleaseFramesToDo() -
+							m_releaseFramesDone;
 	}
-	return( m_frames+actualReleaseFramesToDo()-m_totalFramesPlayed );
+	return m_frames+actualReleaseFramesToDo()-m_totalFramesPlayed;
 }
 
 
@@ -288,7 +302,7 @@ f_cnt_t notePlayHandle::framesLeft( void ) const
 
 bool notePlayHandle::isFromTrack( const track * _track ) const
 {
-	return( m_instrumentTrack == _track || m_bbTrack == _track );
+	return m_instrumentTrack == _track || m_bbTrack == _track;
 }
 
 
@@ -296,6 +310,11 @@ bool notePlayHandle::isFromTrack( const track * _track ) const
 
 void notePlayHandle::noteOff( const f_cnt_t _s )
 {
+	if( m_released )
+	{
+		return;
+	}
+
 	// first note-off all sub-notes
 	for( notePlayHandleVector::iterator it = m_subNotes.begin();
 						it != m_subNotes.end(); ++it )
@@ -305,16 +324,20 @@ void notePlayHandle::noteOff( const f_cnt_t _s )
 
 	// then set some variables indicating release-state
 	m_framesBeforeRelease = _s;
-	m_releaseFramesToDo = tMax<f_cnt_t>( 0, // 10,
-			m_instrumentTrack->m_envWidget->releaseFrames() );
-	// send MIDI-note-off-event
-	m_instrumentTrack->processOutEvent( midiEvent( NOTE_OFF,
-				m_instrumentTrack->m_midiPort->outputChannel(),
+	m_releaseFramesToDo = qMax<f_cnt_t>( 0, // 10,
+			m_instrumentTrack->m_soundShaping.releaseFrames() );
+
+	if( !isBaseNote() || !getInstrumentTrack()->arpeggiatorEnabled() )
+	{
+		// send MIDI-note-off-event
+		m_instrumentTrack->processOutEvent( midiEvent( MidiNoteOff,
+			m_instrumentTrack->getMidiPort()->outputChannel(),
 								key(), 0 ),
 			midiTime::fromFrames( m_framesBeforeRelease,
-						engine::framesPerTact64th() ) );
+						engine::framesPerTick() ) );
+	}
 
-	m_released = TRUE;
+	m_released = true;
 }
 
 
@@ -322,8 +345,8 @@ void notePlayHandle::noteOff( const f_cnt_t _s )
 
 f_cnt_t notePlayHandle::actualReleaseFramesToDo( void ) const
 {
-	return( m_instrumentTrack->m_envWidget->releaseFrames(
-							arpBaseNote() ) );
+	return m_instrumentTrack->m_soundShaping.releaseFrames(/*
+							isArpeggioBaseNote()*/ );
 }
 
 
@@ -336,7 +359,7 @@ void notePlayHandle::setFrames( const f_cnt_t _frames )
 	{
 		m_frames = m_instrumentTrack->beatLen( this );
 	}
-	m_orig_frames = m_frames;
+	m_origFrames = m_frames;
 }
 
 
@@ -344,7 +367,7 @@ void notePlayHandle::setFrames( const f_cnt_t _frames )
 
 float notePlayHandle::volumeLevel( const f_cnt_t _frame )
 {
-	return( m_instrumentTrack->m_envWidget->volumeLevel( this, _frame ) );
+	return m_instrumentTrack->m_soundShaping.volumeLevel( this, _frame );
 }
 
 
@@ -358,7 +381,7 @@ void notePlayHandle::mute( void )
 	{
 		( *it )->mute();
 	}
-	m_muted = TRUE;
+	m_muted = true;
 }
 
 
@@ -375,7 +398,7 @@ int notePlayHandle::index( void ) const
 				dynamic_cast<const notePlayHandle *>( *it );
 		if( nph == NULL ||
 			nph->m_instrumentTrack != m_instrumentTrack ||
-						nph->released() == TRUE )
+						nph->released() == true )
 		{
 			continue;
 		}
@@ -385,7 +408,7 @@ int notePlayHandle::index( void ) const
 		}
 		++idx;
 	}
-	return( idx );
+	return idx;
 }
 
 
@@ -403,12 +426,12 @@ constNotePlayHandleVector notePlayHandle::nphsOfInstrumentTrack(
 		const notePlayHandle * nph =
 				dynamic_cast<const notePlayHandle *>( *it );
 		if( nph != NULL && nph->m_instrumentTrack == _it &&
-			( nph->released() == FALSE || _all_ph == TRUE ) )
+			( nph->released() == false || _all_ph == true ) )
 		{
 			cnphv.push_back( nph );
 		}
 	}
-	return( cnphv );
+	return cnphv;
 }
 
 
@@ -416,7 +439,7 @@ constNotePlayHandleVector notePlayHandle::nphsOfInstrumentTrack(
 
 bool notePlayHandle::operator==( const notePlayHandle & _nph ) const
 {
-	return( length() == _nph.length() &&
+	return length() == _nph.length() &&
 			pos() == _nph.pos() &&
 			key() == _nph.key() &&
 			getVolume() == _nph.getVolume() &&
@@ -427,8 +450,8 @@ bool notePlayHandle::operator==( const notePlayHandle & _nph ) const
 			m_totalFramesPlayed == _nph.m_totalFramesPlayed &&
 			m_released == _nph.m_released &&
 			m_baseNote == _nph.m_baseNote &&
-			m_arpNote == _nph.m_arpNote &&
-			m_muted == _nph.m_muted );
+			m_partOfArpeggio == _nph.m_partOfArpeggio &&
+			m_muted == _nph.m_muted;
 }
 
 
@@ -436,11 +459,12 @@ bool notePlayHandle::operator==( const notePlayHandle & _nph ) const
 
 void notePlayHandle::updateFrequency( void )
 {
-	float pitch = (float)( tone() - m_instrumentTrack->baseTone() +
-			engine::getSongEditor()->masterPitch() ) / 12.0f +
-			(float)( octave() - m_instrumentTrack->baseOctave() )
-			+ m_base_detuning->value() / 12.0f;
-	m_frequency = BASE_FREQ * powf( 2.0f, pitch );
+	const float pitch =
+		( key() - m_instrumentTrack->baseNoteModel()->value() +
+				engine::getSong()->masterPitch() ) / 12.0f;
+	m_frequency = BaseFreq * powf( 2.0f, pitch +
+		m_instrumentTrack->m_pitchModel.value() / ( 100 * 12.0f ) );
+	m_unpitchedFrequency = BaseFreq * powf( 2.0f, pitch );
 
 	for( notePlayHandleVector::iterator it = m_subNotes.begin();
 						it != m_subNotes.end(); ++it )
@@ -456,11 +480,11 @@ void notePlayHandle::processMidiTime( const midiTime & _time )
 {
 	if( _time >= pos() )
 	{
-		int level = detuning()->getAutomationPattern()->valueAt( _time -
-									pos() );
-		if( level != m_base_detuning->level() )
+		const float v = detuning()->getAutomationPattern()->
+						valueAt( _time - pos() );
+		if( !typeInfo<float>::isEqual( v, m_baseDetuning->value() ) )
 		{
-			m_base_detuning->setLevel( level );
+			m_baseDetuning->setValue( v );
 			updateFrequency();
 		}
 	}
@@ -469,17 +493,17 @@ void notePlayHandle::processMidiTime( const midiTime & _time )
 
 
 
-void notePlayHandle::resize( const bpm_t _new_bpm )
+void notePlayHandle::resize( const bpm_t _new_tempo )
 {
-	double completed = m_totalFramesPlayed / (double)m_frames;
-	double new_frames = m_orig_frames * m_orig_bpm / (double)_new_bpm;
+	double completed = m_totalFramesPlayed / (double) m_frames;
+	double new_frames = m_origFrames * m_origTempo / (double) _new_tempo;
 	m_frames = (f_cnt_t)new_frames;
 	m_totalFramesPlayed = (f_cnt_t)( completed * new_frames );
 
 	for( notePlayHandleVector::iterator it = m_subNotes.begin();
 						it != m_subNotes.end(); ++it )
 	{
-		( *it )->resize( _new_bpm );
+		( *it )->resize( _new_tempo );
 	}
 }
 
