@@ -3,7 +3,7 @@
 /*
  * plugin.cpp - implementation of plugin-class including plugin-loader
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,36 +19,27 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
 
-#include "qt3support.h"
-
-#ifdef QT4
-
 #include <QtCore/QDir>
 #include <QtCore/QLibrary>
 #include <QtGui/QMessageBox>
 
-#else
-
-#include <qmessagebox.h>
-#include <qdir.h>
-#include <qlibrary.h>
-
-#endif
-
 
 #include "plugin.h"
+#include "embed.h"
+#include "engine.h"
 #include "mixer.h"
 #include "config_mgr.h"
 #include "dummy_plugin.h"
+#include "automatable_model.h"
 
 
-//static embed::descriptor dummy_embed = { 0, NULL, "" } ;
+static pixmapLoader __dummy_loader;
 
 static plugin::descriptor dummy_plugin_descriptor =
 {
@@ -57,21 +48,19 @@ static plugin::descriptor dummy_plugin_descriptor =
 	QT_TRANSLATE_NOOP( "pluginBrowser", "no description" ),
 	"Tobias Doerffel <tobydox/at/users.sf.net>",
 	0x0100,
-	plugin::UNDEFINED,
+	plugin::Undefined,
+	&__dummy_loader,
 	NULL
 } ;
 
 
 
-plugin::plugin( const descriptor * _descriptor, engine * _engine ) :
-	journallingObject( _engine ),
+
+plugin::plugin( const descriptor * _descriptor, model * _parent ) :
+	journallingObject(),
+	model( _parent ),
 	m_descriptor( _descriptor )
 {
-	if( dummy_plugin_descriptor.logo == NULL )
-	{
-		dummy_plugin_descriptor.logo = new QPixmap();
-	}
-
 	if( m_descriptor == NULL )
 	{
 		m_descriptor = &dummy_plugin_descriptor;
@@ -88,162 +77,102 @@ plugin::~plugin()
 
 
 
-void plugin::setParameter( const QString &, const QString & )
+void plugin::loadFile( const QString & )
 {
 }
 
 
 
 
-QString plugin::getParameter( const QString & )
+automatableModel * plugin::getChildModel( const QString & )
 {
-	return( "" );
+	static floatModel fm;
+	return &fm;
 }
 
 
 
 
-plugin * plugin::instantiate( const QString & _plugin_name, void * _data )
+plugin * plugin::instantiate( const QString & _plugin_name, model * _parent,
+								void * _data )
 {
-/*#ifdef HAVE_DLFCN_H
-	void * handle = dlopen( QString( "lib" + _plugin_name + ".so" ).
-#ifdef QT4
-					toAscii().constData()
-#else
-					ascii()
-#endif
-					, RTLD_NOW );*/
 	QLibrary plugin_lib( configManager::inst()->pluginDir() +
 								_plugin_name );
-	if( /*handle == NULL*/ plugin_lib.load() == FALSE )
+	if( plugin_lib.load() == FALSE )
 	{
-		QMessageBox::information( NULL,
-					mixer::tr( "Plugin not found" ),
-					mixer::tr( "The %1-plugin wasn't found!"
-							).arg( _plugin_name ),
+		if( engine::hasGUI() )
+		{
+			QMessageBox::information( NULL,
+				tr( "Plugin not found" ),
+				tr( "The plugin \"%1\" wasn't found "
+					"or could not be loaded!\n"
+					"Reason: \"%2\"" ).arg( _plugin_name ).
+						arg( plugin_lib.errorString() ),
 					QMessageBox::Ok |
 						QMessageBox::Default );
+		}
 		return( new dummyPlugin() );
 	}
 	instantiationHook inst_hook = ( instantiationHook ) plugin_lib.resolve(
 							"lmms_plugin_main" );
-/*	dlerror();
-	instantiationHook inst_hook = ( instantiationHook )( dlsym( handle,
-							"lmms_plugin_main" ) );
-	char * error = dlerror();
-	if( error != NULL )
-	{
-		printf( "%s\n", error );*/
 	if( inst_hook == NULL )
 	{
-		QMessageBox::information( NULL,
-				mixer::tr( "Error while loading plugin" ),
-				mixer::tr( "Failed loading plugin \"%1\"!"
+		if( engine::hasGUI() )
+		{
+			QMessageBox::information( NULL,
+				tr( "Error while loading plugin" ),
+				tr( "Failed to load plugin \"%1\"!"
 							).arg( _plugin_name ),
 					QMessageBox::Ok |
 						QMessageBox::Default );
+		}
 		return( new dummyPlugin() );
 	}
-#ifndef QT4
-	plugin_lib.setAutoUnload( FALSE );
-#endif
-	plugin * inst = inst_hook( _data );
-	//dlclose( handle );
-	return( inst );
-/*#endif
-	return( new dummyPlugin() );*/
+	plugin * inst = inst_hook( _parent, _data );
+	return inst;
 }
 
 
 
 
-void plugin::getDescriptorsOfAvailPlugins( vvector<descriptor> & _plugin_descs )
+void plugin::getDescriptorsOfAvailPlugins( QVector<descriptor> & _plugin_descs )
 {
 	QDir directory( configManager::inst()->pluginDir() );
-#ifdef QT4
+#ifdef LMMS_BUILD_WIN32
+	QFileInfoList list = directory.entryInfoList(
+						QStringList( "*.dll" ) );
+#else
 	QFileInfoList list = directory.entryInfoList(
 						QStringList( "lib*.so" ) );
-#else
-	const QFileInfoList * lp = directory.entryInfoList( "lib*.so" );
-	// if directory doesn't exist or isn't readable, we get NULL which would
-	// crash LMMS...
-	if( lp == NULL )
-	{
-		return;
-	}
-	QFileInfoList list = *lp;
 #endif
 	for( QFileInfoList::iterator file = list.begin();
 						file != list.end(); ++file )
 	{
-#ifdef QT4
 		const QFileInfo & f = *file;
-#else
-		const QFileInfo & f = **file;
-#endif
 		QLibrary plugin_lib( f.absoluteFilePath() );
 		if( plugin_lib.load() == FALSE ||
 			plugin_lib.resolve( "lmms_plugin_main" ) == NULL )
 		{
 			continue;
 		}
-#ifndef QT4
-		plugin_lib.setAutoUnload( FALSE );
-#endif
-/*
-		void * handle = dlopen( f.absoluteFilePath().
-#ifdef QT4
-							toAscii().constData(),
-#else
-							ascii(),
-#endif
-								RTLD_NOW );
-		char * msg = dlerror();
-		if( msg != NULL || handle == NULL )
-		{
-			printf( "plugin-loader: %s\n", msg );
-			continue;
-		}
-		void * foo = dlsym( handle, "lmms_plugin_main" );
-		msg = dlerror();
-		if( msg != NULL || foo == NULL )
-		{
-			printf( "plugin-loader: %s\n", msg );
-			continue;
-		}*/
 		QString desc_name = f.fileName().section( '.', 0, 0 ) +
 							"_plugin_descriptor";
 		if( desc_name.left( 3 ) == "lib" )
 		{
 			desc_name = desc_name.mid( 3 );
 		}
-		descriptor * plugin_desc = (descriptor *) plugin_lib.resolve(
-						desc_name.
-#ifdef QT4
-							toAscii().constData()
-#else
-							ascii()
-#endif
-								);
+		descriptor * plugin_desc =
+			(descriptor *) plugin_lib.resolve(
+					desc_name.toAscii().constData() );
 		if( plugin_desc == NULL )
 		{
+			printf( "LMMS-plugin %s does not have a "
+				"plugin-descriptor named %s!\n",
+				f.absoluteFilePath().toAscii().constData(),
+					desc_name.toAscii().constData() );
 			continue;
 		}
-/*			(descriptor *) dlsym( handle, desc_name.
-#ifdef QT4
-					      toAscii().constData()
-#else
-					      ascii()
-#endif
-					      );
-		msg = dlerror();
-		if( msg != NULL || plugin_desc == NULL )
-		{
-			printf( "plugin-loader: %s\n", msg );
-			continue;
-		}*/
 		_plugin_descs.push_back( *plugin_desc );
-		//dlclose( handle );
 	}
 
 }
@@ -251,4 +180,50 @@ void plugin::getDescriptorsOfAvailPlugins( vvector<descriptor> & _plugin_descs )
 
 
 
+pluginView * plugin::createView( QWidget * _parent )
+{
+	pluginView * pv = instantiateView( _parent );
+	if( pv != NULL )
+	{
+		pv->setModel( this );
+	}
+	return pv;
+}
+
+
+
+plugin::descriptor::subPluginFeatures::key::key(
+						const QDomElement & _key ) :
+	desc( NULL ),
+	name( _key.attribute( "key" ) ),
+	attributes()
+{
+	QDomNodeList l = _key.elementsByTagName( "attribute" );
+	for( int i = 0; !l.item( i ).isNull(); ++i )
+	{
+		QDomElement e = l.item( i ).toElement();
+		attributes[e.attribute( "name" )] = e.attribute( "value" );
+	}
+		
+}
+
+
+
+
+QDomElement plugin::descriptor::subPluginFeatures::key::saveXML(
+						QDomDocument & _doc ) const
+{
+	QDomElement e = _doc.createElement( "key" );
+	for( attributeMap::const_iterator it = attributes.begin();
+						it != attributes.end(); ++it )
+	{
+		QDomElement a = _doc.createElement( "attribute" );
+		a.setAttribute( "name", it.key() );
+		a.setAttribute( "value", it.value() );
+		e.appendChild( a );
+	}
+	return e;
+}
+
+
 #endif
