@@ -3,7 +3,7 @@
 /*
  * sample_play_handle.cpp - implementation of class samplePlayHandle
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -39,11 +39,12 @@
 samplePlayHandle::samplePlayHandle( const QString & _sample_file ) :
 	playHandle( SamplePlayHandle ),
 	m_sampleBuffer( new sampleBuffer( _sample_file ) ),
-	m_doneMayReturnTrue( TRUE ),
+	m_doneMayReturnTrue( true ),
 	m_frame( 0 ),
-	m_audioPort( new audioPort( "samplePlayHandle" ) ),
-	m_ownAudioPort( TRUE ),
-	m_volume( 1.0f ),
+	m_audioPort( new audioPort( "samplePlayHandle", false ) ),
+	m_ownAudioPort( true ),
+	m_defaultVolumeModel( DefaultVolume, MinVolume, MaxVolume, 1 ),
+	m_volumeModel( &m_defaultVolumeModel ),
 	m_track( NULL ),
 	m_bbTrack( NULL )
 {
@@ -55,11 +56,12 @@ samplePlayHandle::samplePlayHandle( const QString & _sample_file ) :
 samplePlayHandle::samplePlayHandle( sampleBuffer * _sample_buffer ) :
 	playHandle( SamplePlayHandle ),
 	m_sampleBuffer( sharedObject::ref( _sample_buffer ) ),
-	m_doneMayReturnTrue( TRUE ),
+	m_doneMayReturnTrue( true ),
 	m_frame( 0 ),
-	m_audioPort( new audioPort( "samplePlayHandle" ) ),
-	m_ownAudioPort( TRUE ),
-	m_volume( 1.0f ),
+	m_audioPort( new audioPort( "samplePlayHandle", false ) ),
+	m_ownAudioPort( true ),
+	m_defaultVolumeModel( DefaultVolume, MinVolume, MaxVolume, 1 ),
+	m_volumeModel( &m_defaultVolumeModel ),
 	m_track( NULL ),
 	m_bbTrack( NULL )
 {
@@ -71,11 +73,12 @@ samplePlayHandle::samplePlayHandle( sampleBuffer * _sample_buffer ) :
 samplePlayHandle::samplePlayHandle( sampleTCO * _tco ) :
 	playHandle( SamplePlayHandle ),
 	m_sampleBuffer( sharedObject::ref( _tco->getSampleBuffer() ) ),
-	m_doneMayReturnTrue( TRUE ),
+	m_doneMayReturnTrue( true ),
 	m_frame( 0 ),
 	m_audioPort( ( (sampleTrack *)_tco->getTrack() )->getAudioPort() ),
-	m_ownAudioPort( FALSE ),
-	m_volume( 1.0f ),
+	m_ownAudioPort( false ),
+	m_defaultVolumeModel( DefaultVolume, MinVolume, MaxVolume, 1 ),
+	m_volumeModel( &m_defaultVolumeModel ),
 	m_track( _tco->getTrack() ),
 	m_bbTrack( NULL )
 {
@@ -87,11 +90,12 @@ samplePlayHandle::samplePlayHandle( sampleTCO * _tco ) :
 samplePlayHandle::samplePlayHandle( pattern * _pattern ) :
 	playHandle( SamplePlayHandle ),
 	m_sampleBuffer( sharedObject::ref( _pattern->getFrozenPattern() ) ),
-	m_doneMayReturnTrue( TRUE ),
+	m_doneMayReturnTrue( true ),
 	m_frame( 0 ),
 	m_audioPort( _pattern->getInstrumentTrack()->getAudioPort() ),
-	m_ownAudioPort( FALSE ),
-	m_volume( 1.0f ),
+	m_ownAudioPort( false ),
+	m_defaultVolumeModel( DefaultVolume, MinVolume, MaxVolume, 1 ),
+	m_volumeModel( &m_defaultVolumeModel ),
 	m_track( _pattern->getInstrumentTrack() ),
 	m_bbTrack( NULL )
 {
@@ -112,7 +116,7 @@ samplePlayHandle::~samplePlayHandle()
 
 
 
-void samplePlayHandle::play( bool /* _try_parallelizing */ )
+void samplePlayHandle::play( sampleFrame * _working_buffer )
 {
 	//play( 0, _try_parallelizing );
 	if( framesDone() >= totalFrames() )
@@ -121,20 +125,16 @@ void samplePlayHandle::play( bool /* _try_parallelizing */ )
 	}
 
 	const fpp_t frames = engine::getMixer()->framesPerPeriod();
-	if( !( m_track && m_track->muted() )
-				&& !( m_bbTrack && m_bbTrack->muted() ) )
+	if( !( m_track && m_track->isMuted() )
+				&& !( m_bbTrack && m_bbTrack->isMuted() ) )
 	{
-		sampleFrame * buf = new sampleFrame[frames];
-		volumeVector v = { { m_volume, m_volume
-#ifndef DISABLE_SURROUND
-						, m_volume, m_volume
-#endif
-				} } ;
-		m_sampleBuffer->play( buf, &m_state, frames );
-		engine::getMixer()->bufferToPort( buf, frames, offset(), v,
-								m_audioPort );
-
-		delete[] buf;
+		stereoVolumeVector v =
+			{ { m_volumeModel->value() / DefaultVolume,
+				m_volumeModel->value() / DefaultVolume } };
+		m_sampleBuffer->play( _working_buffer, &m_state, frames,
+								BaseFreq );
+		engine::getMixer()->bufferToPort( _working_buffer, frames,
+						offset(), v, m_audioPort );
 	}
 
 	m_frame += frames;
@@ -145,7 +145,7 @@ void samplePlayHandle::play( bool /* _try_parallelizing */ )
 
 bool samplePlayHandle::done( void ) const
 {
-	return( framesDone() >= totalFrames() && m_doneMayReturnTrue == TRUE );
+	return( framesDone() >= totalFrames() && m_doneMayReturnTrue == true );
 }
 
 
@@ -162,25 +162,11 @@ bool samplePlayHandle::isFromTrack( const track * _track ) const
 f_cnt_t samplePlayHandle::totalFrames( void ) const
 {
 	return( ( m_sampleBuffer->endFrame() - m_sampleBuffer->startFrame() ) *
-				( engine::getMixer()->sampleRate() /
-					SAMPLE_RATES[DEFAULT_QUALITY_LEVEL] ) );
+			( engine::getMixer()->processingSampleRate() /
+				engine::getMixer()->baseSampleRate() ) );
 }
 
 
 
 
-void samplePlayHandle::setVolume( float _new_volume )
-{
-	if( _new_volume <= MAX_VOLUME )
-	{
-		m_volume = _new_volume / 100.0f;
-	}
-}
-
-
-
-
-#include "sample_play_handle.moc"
-
-
 #endif
