@@ -3,7 +3,7 @@
 /*
  * timeline.cpp - class timeLine, representing a time-line with position marker
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -25,27 +25,13 @@
  */
 
 
-#include "qt3support.h"
-
-#ifdef QT4
-
-#include <Qt/QtXml>
+#include <QtXml/QDomElement>
 #include <QtCore/QTimer>
 #include <QtGui/QApplication>
 #include <QtGui/QLayout>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPainter>
 
-#else
-
-#include <qpainter.h>
-#include <qapplication.h>
-#include <qlayout.h>
-#include <qtimer.h>
-#include <qdom.h>
-
-#endif
-
 
 #include "timeline.h"
 #include "embed.h"
@@ -60,16 +46,15 @@
 QPixmap * timeLine::s_timeLinePixmap = NULL;
 QPixmap * timeLine::s_posMarkerPixmap = NULL;
 QPixmap * timeLine::s_loopPointPixmap = NULL;
-QPixmap * timeLine::s_loopPointDisabledPixmap = NULL;
 
 
 timeLine::timeLine( const int _xoff, const int _yoff, const float _ppt,
-			songEditor::playPos & _pos, const midiTime & _begin,
+			song::playPos & _pos, const midiTime & _begin,
 							QWidget * _parent ) :
 	QWidget( _parent ),
-	m_autoScroll( AUTOSCROLL_ENABLED ),
-	m_loopPoints( LOOP_POINTS_DISABLED ),
-	m_behaviourAtStop( BACK_TO_ZERO ),
+	m_autoScroll( AutoScrollEnabled ),
+	m_loopPoints( LoopPointsDisabled ),
+	m_behaviourAtStop( BackToZero ),
 	m_changedPosition( TRUE ),
 	m_xOffset( _xoff ),
 	m_posMarkerX( 0 ),
@@ -78,11 +63,11 @@ timeLine::timeLine( const int _xoff, const int _yoff, const float _ppt,
 	m_begin( _begin ),
 	m_savedPos( -1 ),
 	m_hint( NULL ),
-	m_action( NONE ),
+	m_action( NoAction ),
 	m_moveXOff( 0 )
 {
 	m_loopPos[0] = 0;
-	m_loopPos[1] = 64;
+	m_loopPos[1] = DefaultTicksPerTact;
 
 	if( s_timeLinePixmap == NULL )
 	{
@@ -100,21 +85,12 @@ timeLine::timeLine( const int _xoff, const int _yoff, const float _ppt,
 							"loop_point" ) );
 	}
 
-	if( s_loopPointDisabledPixmap == NULL )
-	{
-		s_loopPointDisabledPixmap = new QPixmap( embed::getIconPixmap(
-						"loop_point_disabled" ) );
-	}
-
+	setAttribute( Qt::WA_OpaquePaintEvent, true );
 	move( 0, _yoff );
 	setFixedHeight( s_timeLinePixmap->height() );
 
 	m_xOffset -= s_posMarkerPixmap->width() / 2;
 
-#ifndef QT4
-	setBackgroundMode( Qt::NoBackground );
-#endif
-
 	m_pos.m_timeLine = this;
 
 	QTimer * update_timer = new QTimer( this );
@@ -192,7 +168,7 @@ void timeLine::loadSettings( const QDomElement & _this )
 {
 	m_loopPos[0] = _this.attribute( "lp0pos" ).toInt();
 	m_loopPos[1] = _this.attribute( "lp1pos" ).toInt();
-	m_loopPoints = static_cast<loopPointStates>(
+	m_loopPoints = static_cast<LoopPointStates>(
 					_this.attribute( "lpstate" ).toInt() );
 	update();
 	emit loopPointStateLoaded( m_loopPoints );
@@ -209,10 +185,7 @@ void timeLine::updatePosition( const midiTime & )
 	{
 		m_posMarkerX = new_x;
 		m_changedPosition = TRUE;
-		if( m_autoScroll == AUTOSCROLL_ENABLED )
-		{
-			emit positionChanged( m_pos );
-		}
+		emit positionChanged( m_pos );
 		update();
 	}
 }
@@ -222,7 +195,7 @@ void timeLine::updatePosition( const midiTime & )
 
 void timeLine::toggleAutoScroll( int _n )
 {
-	m_autoScroll = static_cast<autoScrollStates>( _n );
+	m_autoScroll = static_cast<AutoScrollStates>( _n );
 }
 
 
@@ -230,7 +203,7 @@ void timeLine::toggleAutoScroll( int _n )
 
 void timeLine::toggleLoopPoints( int _n )
 {
-	m_loopPoints = static_cast<loopPointStates>( _n );
+	m_loopPoints = static_cast<LoopPointStates>( _n );
 	update();
 }
 
@@ -239,7 +212,7 @@ void timeLine::toggleLoopPoints( int _n )
 
 void timeLine::toggleBehaviourAtStop( int _n )
 {
-	m_behaviourAtStop = static_cast<behaviourAtStopStates>( _n );
+	m_behaviourAtStop = static_cast<BehaviourAtStopStates>( _n );
 }
 
 
@@ -247,54 +220,50 @@ void timeLine::toggleBehaviourAtStop( int _n )
 
 void timeLine::paintEvent( QPaintEvent * )
 {
-#ifdef QT4
 	QPainter p( this );
-#else
-	QPixmap draw_pm( rect().size() );
 
-	QPainter p( &draw_pm, this );
-#endif
+	QColor bg_color = QApplication::palette().color( QPalette::Active,
+							QPalette::Background );
+	QLinearGradient g( 0, 0, 0, height() );
+	g.setColorAt( 0, bg_color.lighter( 150 ) );
+	g.setColorAt( 1, bg_color.darker( 150 ) );
+	p.fillRect( 0, 0, width(), height(), g );
 
-	for( int x = 0; x < width(); x += s_timeLinePixmap->width() )
-	{
-		p.drawPixmap( x, 0, *s_timeLinePixmap );
-	}
 	p.setClipRect( m_xOffset, 0, width() - m_xOffset, height() );
 	p.setPen( QColor( 0, 0, 0 ) );
 
-	const QPixmap & lpoint = loopPointsEnabled() ?
-						*s_loopPointPixmap :
-						*s_loopPointDisabledPixmap;
-	p.drawPixmap( markerX( loopBegin() ), 7, lpoint );
-	p.drawPixmap( markerX( loopEnd() ), 7, lpoint );
+	p.setOpacity( loopPointsEnabled() ? 0.9 : 0.2 );
+	p.drawPixmap( markerX( loopBegin() )+2, 2, *s_loopPointPixmap );
+	p.drawPixmap( markerX( loopEnd() )+2, 2, *s_loopPointPixmap );
+	p.setOpacity( 1.0 );
 
 
 	tact tact_num = m_begin.getTact();
 	int x = m_xOffset + s_posMarkerPixmap->width() / 2 -
-			( ( static_cast<Sint32>( m_begin * m_ppt ) / 64 ) %
+			( ( static_cast<Sint32>( m_begin * m_ppt ) /
+						midiTime::ticksPerTact() ) %
 						static_cast<int>( m_ppt ) );
 
+	p.setPen( QColor( 192, 192, 192 ) );
 	for( int i = 0; x + i * m_ppt < width(); ++i )
 	{
+		const int cx = x + qRound( i * m_ppt );
+		p.drawLine( cx, 5, cx, height() - 6 );
 		++tact_num;
 		if( ( tact_num - 1 ) %
-			tMax( 1, static_cast<int>( 64.0f / m_ppt ) ) == 0 )
+			qMax( 1, qRound( 1.0f / 3.0f *
+				midiTime::ticksPerTact() / m_ppt ) ) == 0 )
 		{
-			p.setPen( QColor( 224, 224, 224 ) );
-			p.drawText( x + static_cast<int>( i * m_ppt ) + 1, 15,
-						QString::number( tact_num ) );
-			p.setPen( QColor( 0, 0, 0 ) );
-			p.drawText( x + static_cast<int>( i * m_ppt ), 14,
-						QString::number( tact_num ) );
+			const QString s = QString::number( tact_num );
+			p.drawText( cx + qRound( ( m_ppt - p.fontMetrics().
+							width( s ) ) / 2 ),
+				height() - p.fontMetrics().height() / 2, s );
 		}
 	}
 
-	p.drawPixmap( m_posMarkerX, 4, *s_posMarkerPixmap );
-
-#ifndef QT4
-	// and blit all the drawn stuff on the screen...
-	bitBlt( this, rect().topLeft(), &draw_pm );
-#endif
+	p.setOpacity( 0.6 );
+	p.drawPixmap( m_posMarkerX, height() - s_posMarkerPixmap->height(),
+							*s_posMarkerPixmap );
 }
 
 
@@ -308,7 +277,7 @@ void timeLine::mousePressEvent( QMouseEvent * _me )
 	}
 	if( _me->button() == Qt::LeftButton )
 	{
-		m_action = MOVE_POS_MARKER;
+		m_action = MovePositionMarker;
 		if( _me->x() - m_xOffset < s_posMarkerPixmap->width() )
 		{
 			m_moveXOff = _me->x() - m_xOffset;
@@ -321,19 +290,20 @@ void timeLine::mousePressEvent( QMouseEvent * _me )
 	else
 	{
 		const midiTime t = m_begin +
-				static_cast<Sint32>( _me->x() * 64 / m_ppt );
-		m_action = MOVE_LOOP_BEGIN;
+				static_cast<Sint32>( _me->x() *
+					midiTime::ticksPerTact() / m_ppt );
+		m_action = MoveLoopBegin;
 		if( m_loopPos[0] > m_loopPos[1]  )
 		{
 			qSwap( m_loopPos[0], m_loopPos[1] );
 		}
 		if( _me->button() == Qt::RightButton )
 		{
-			m_action = MOVE_LOOP_END;
+			m_action = MoveLoopEnd;
 		}
-		m_loopPos[( m_action == MOVE_LOOP_BEGIN ) ? 0 : 1] = t;
+		m_loopPos[( m_action == MoveLoopBegin ) ? 0 : 1] = t;
 	}
-	if( m_action == MOVE_LOOP_BEGIN || m_action == MOVE_LOOP_END )
+	if( m_action == MoveLoopBegin || m_action == MoveLoopEnd )
 	{
 		delete m_hint;
 		m_hint = textFloat::displayMessage( tr( "Hint" ),
@@ -349,22 +319,22 @@ void timeLine::mousePressEvent( QMouseEvent * _me )
 
 void timeLine::mouseMoveEvent( QMouseEvent * _me )
 {
-	const midiTime t = m_begin + static_cast<Sint32>( tMax( _me->x() -
-				    m_xOffset - m_moveXOff, 0 ) * 64 / m_ppt );
+	const midiTime t = m_begin + static_cast<Sint32>( qMax( _me->x() -
+				    m_xOffset - m_moveXOff, 0 ) *
+					midiTime::ticksPerTact() / m_ppt );
 	switch( m_action )
 	{
-		case MOVE_POS_MARKER:
-			m_pos.setTact( t.getTact() );
-			m_pos.setTact64th( t.getTact64th() );
+		case MovePositionMarker:
+			m_pos.setTicks( t.getTicks() );
 			m_pos.setCurrentFrame( 0 );
 			updatePosition();
 			break;
 
-		case MOVE_LOOP_BEGIN:
-		case MOVE_LOOP_END:
+		case MoveLoopBegin:
+		case MoveLoopEnd:
 		{
-			const Uint8 i = m_action - MOVE_LOOP_BEGIN;
-			if( engine::getMainWindow()->isCtrlPressed() == TRUE )
+			const Uint8 i = m_action - MoveLoopBegin;
+			if( _me->modifiers() & Qt::ControlModifier )
 			{
 				// no ctrl-press-hint when having ctrl pressed
 				delete m_hint;
@@ -375,6 +345,16 @@ void timeLine::mouseMoveEvent( QMouseEvent * _me )
 			{
 				m_loopPos[i] = t.toNearestTact();
 			}
+			// Catch begin == end
+			if( m_loopPos[0] == m_loopPos[1] )
+			{
+				// Note, swap 1 and 0 below and the behavior "skips" the other
+				// marking instead of pushing it.
+				if( m_action == MoveLoopBegin ) 
+					m_loopPos[0] -= midiTime::ticksPerTact();
+				else
+					m_loopPos[1] += midiTime::ticksPerTact();
+			}
 			update();
 			break;
 		}
@@ -391,13 +371,13 @@ void timeLine::mouseReleaseEvent( QMouseEvent * _me )
 {
 	delete m_hint;
 	m_hint = NULL;
-	m_action = NONE;
+	m_action = NoAction;
 }
 
 
 
 
-#include "timeline.moc"
+#include "moc_timeline.cxx"
 
 
 #endif
