@@ -3,7 +3,7 @@
 /*
  * bb_track.cpp - implementation of class bbTrack and bbTCO
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -25,52 +25,35 @@
  */
 
 
-#include "qt3support.h"
-
-#ifdef QT4
-
-#include <Qt/QtXml>
+#include <QtXml/QDomElement>
 #include <QtGui/QColorDialog>
 #include <QtGui/QMenu>
 #include <QtGui/QPainter>
 
-#else
 
-#include <qdom.h>
-#include <qpainter.h>
-#include <qcolordialog.h>
-#include <qpopupmenu.h>
-
-#endif
-
-
-#include "bb_track.h"
 #include "bb_editor.h"
+#include "bb_track.h"
+#include "bb_track_container.h"
 #include "embed.h"
 #include "engine.h"
 #include "gui_templates.h"
 #include "mixer.h"
-#include "name_label.h"
 #include "rename_dialog.h"
+#include "song.h"
 #include "song_editor.h"
 #include "templates.h"
+#include "track_label_button.h"
 
 
 
 bbTrack::infoMap bbTrack::s_infoMap;
 
 
-bbTCO::bbTCO( track * _track, const QColor & _c ) :
+bbTCO::bbTCO( track * _track, unsigned int _color ) :
 	trackContentObject( _track ),
-	m_name( ( dynamic_cast<bbTrack *>( _track ) != NULL ) ?
-		dynamic_cast<bbTrack *>( _track )->trackLabel()->text() :
-								QString( "" ) ),
-	m_color( _c.isValid() ? _c : QColor( 64, 128, 255 ) )
+	m_color( _color > 0 ? _color : qRgb( 64, 128, 255 ) )
 {
-#ifndef QT4
-	setBackgroundMode( Qt::NoBackground );
-#endif
-	tact t = engine::getBBEditor()->lengthOfBB(
+	tact t = engine::getBBTrackContainer()->lengthOfBB(
 					bbTrack::numOfBBTrack( getTrack() ) );
 	if( t > 0 )
 	{
@@ -90,34 +73,91 @@ bbTCO::~bbTCO()
 
 
 
-void bbTCO::constructContextMenu( QMenu * _cm )
+void bbTCO::saveSettings( QDomDocument & _doc, QDomElement & _this )
+{
+	_this.setAttribute( "name", name() );
+	if( _this.parentNode().nodeName() == "clipboard" )
+	{
+		_this.setAttribute( "pos", -1 );
+	}
+	else
+	{
+		_this.setAttribute( "pos", startPosition() );
+	}
+	_this.setAttribute( "len", length() );
+	_this.setAttribute( "muted", isMuted() );
+	_this.setAttribute( "color", m_color );
+}
+
+
+
+
+void bbTCO::loadSettings( const QDomElement & _this )
+{
+	setName( _this.attribute( "name" ) );
+	if( _this.attribute( "pos" ).toInt() >= 0 )
+	{
+		movePosition( _this.attribute( "pos" ).toInt() );
+	}
+	changeLength( _this.attribute( "len" ).toInt() );
+	if( _this.attribute( "muted" ).toInt() != isMuted() )
+	{
+		toggleMute();
+	}
+
+	if( _this.attribute( "color" ).toUInt() != 0 )
+	{
+		m_color = _this.attribute( "color" ).toUInt();
+	}
+}
+
+
+
+
+trackContentObjectView * bbTCO::createView( trackView * _tv )
+{
+	return( new bbTCOView( this, _tv ) );
+}
+
+
+
+
+
+
+
+
+
+
+bbTCOView::bbTCOView( trackContentObject * _tco, trackView * _tv ) :
+	trackContentObjectView( _tco, _tv ),
+	m_bbTCO( dynamic_cast<bbTCO *>( _tco ) )
+{
+}
+
+
+
+
+bbTCOView::~bbTCOView()
+{
+}
+
+
+
+
+void bbTCOView::constructContextMenu( QMenu * _cm )
 {
-#ifdef QT4
 	QAction * a = new QAction( embed::getIconPixmap( "bb_track" ),
-					tr( "Open in Beat+Baseline-Editor" ),
+					tr( "Open in Beat+Bassline-Editor" ),
 					_cm );
 	_cm->insertAction( _cm->actions()[0], a );
-	connect( a, SIGNAL( triggered( bool ) ), this,
-					SLOT( openInBBEditor( bool ) ) );
-#else
-	_cm->insertItem( embed::getIconPixmap( "bb_track" ),
-					tr( "Open in Beat+Baseline-Editor" ),
-					this, SLOT( openInBBEditor() ),
-								0, -1, 0 );
-#endif
-#ifdef QT4
+	connect( a, SIGNAL( triggered( bool ) ),
+			this, SLOT( openInBBEditor() ) );
 	_cm->insertSeparator( _cm->actions()[1] );
-#else
-	_cm->insertSeparator( 1 );
-#endif
-#ifdef QT4
 	_cm->addSeparator();
-#else
-	_cm->insertSeparator();
-#endif
 	_cm->addAction( embed::getIconPixmap( "reload" ), tr( "Reset name" ),
 						this, SLOT( resetName() ) );
-	_cm->addAction( embed::getIconPixmap( "rename" ), tr( "Change name" ),
+	_cm->addAction( embed::getIconPixmap( "edit_rename" ),
+						tr( "Change name" ),
 						this, SLOT( changeName() ) );
 	_cm->addAction( embed::getIconPixmap( "colorize" ),
 			tr( "Change color" ), this, SLOT( changeColor() ) );
@@ -126,7 +166,7 @@ void bbTCO::constructContextMenu( QMenu * _cm )
 
 
 
-void bbTCO::mouseDoubleClickEvent( QMouseEvent * )
+void bbTCOView::mouseDoubleClickEvent( QMouseEvent * )
 {
 	openInBBEditor();
 }
@@ -134,42 +174,28 @@ void bbTCO::mouseDoubleClickEvent( QMouseEvent * )
 
 
 
-void bbTCO::paintEvent( QPaintEvent * )
+void bbTCOView::paintEvent( QPaintEvent * )
 {
-	QColor col = m_color;
-	if( getTrack()->muted() || muted() )
+	QColor col( m_bbTCO->m_color );
+	if( m_bbTCO->getTrack()->isMuted() || m_bbTCO->isMuted() )
 	{
 		col = QColor( 160, 160, 160 );
 	}
 	if( isSelected() == TRUE )
 	{
-		col = QColor( tMax( col.red() - 128, 0 ),
-					tMax( col.green() - 128, 0 ), 255 );
+		col = QColor( qMax( col.red() - 128, 0 ),
+					qMax( col.green() - 128, 0 ), 255 );
 	}
-#ifdef QT4
 	QPainter p( this );
 
 	QLinearGradient lingrad( 0, 0, 0, height() );
 	lingrad.setColorAt( 0, col.light( 130 ) );
 	lingrad.setColorAt( 1, col.light( 70 ) );
 	p.fillRect( rect(), lingrad );
-#else
-	// create pixmap for whole widget
-	QPixmap pm( rect().size() );
-	// and a painter for it
-	QPainter p( &pm );
 
-	// COOL gradient ;-)
-	for( int y = 1; y < height() - 1; ++y )
-	{
-		p.setPen( col.light( 130 - y * 60 / height() ) );
-		p.drawLine( 1, y, width() - 1, y );
-	}
-#endif
-
-	tact t = engine::getBBEditor()->lengthOfBB( bbTrack::numOfBBTrack(
-								getTrack() ) );
-	if( length() > 64 && t > 0 )
+	tact t = engine::getBBTrackContainer()->lengthOfBB(
+				bbTrack::numOfBBTrack( m_bbTCO->getTrack() ) );
+	if( m_bbTCO->length() > midiTime::ticksPerTact() && t > 0 )
 	{
 		for( int x = static_cast<int>( t * pixelsPerTact() );
 								x < width()-2;
@@ -183,75 +209,26 @@ void bbTCO::paintEvent( QPaintEvent * )
 	}
 
 	p.setPen( col.dark() );
-#ifndef QT3
 	p.drawRect( 0, 0, rect().right(), rect().bottom() );
-#else
-	p.drawRect( rect() );
-#endif
 
 	p.setFont( pointSize<7>( p.font() ) );
 	p.setPen( QColor( 0, 0, 0 ) );
-	p.drawText( 2, p.fontMetrics().height() - 1, m_name );
+	p.drawText( 2, p.fontMetrics().height() - 1, m_bbTCO->name() );
 
-	if( muted() )
+	if( m_bbTCO->isMuted() )
 	{
 		p.drawPixmap( 3, p.fontMetrics().height() + 1,
 				embed::getIconPixmap( "muted", 16, 16 ) );
 	}
-
-#ifndef QT4
-	bitBlt( this, rect().topLeft(), &pm );
-#endif
 }
 
 
 
 
-void bbTCO::saveSettings( QDomDocument & _doc, QDomElement & _this )
+void bbTCOView::openInBBEditor( void )
 {
-	_this.setAttribute( "name", m_name );
-	if( _this.parentNode().nodeName() == "clipboard" )
-	{
-		_this.setAttribute( "pos", -1 );
-	}
-	else
-	{
-		_this.setAttribute( "pos", startPosition() );
-	}
-	_this.setAttribute( "len", length() );
-	_this.setAttribute( "muted", muted() );
-	_this.setAttribute( "color", m_color.rgb() );
-}
-
-
-
-
-void bbTCO::loadSettings( const QDomElement & _this )
-{
-	m_name = _this.attribute( "name" );
-	if( _this.attribute( "pos" ).toInt() >= 0 )
-	{
-		movePosition( _this.attribute( "pos" ).toInt() );
-	}
-	changeLength( _this.attribute( "len" ).toInt() );
-	if( _this.attribute( "muted" ).toInt() != muted() )
-	{
-		toggleMute();
-	}
-
-	if( _this.attribute( "color" ).toUInt() != 0 )
-	{
-		m_color.setRgb( _this.attribute( "color" ).toUInt() );
-	}
-}
-
-
-
-
-void bbTCO::openInBBEditor( bool )
-{
-	engine::getBBEditor()->setCurrentBB( bbTrack::numOfBBTrack(
-								getTrack() ) );
+	engine::getBBTrackContainer()->setCurrentBB( bbTrack::numOfBBTrack(
+							m_bbTCO->getTrack() ) );
 	engine::getBBEditor()->show();
 	engine::getBBEditor()->setFocus();
 }
@@ -259,54 +236,44 @@ void bbTCO::openInBBEditor( bool )
 
 
 
-void bbTCO::openInBBEditor( void )
+void bbTCOView::resetName( void )
 {
-	openInBBEditor( FALSE );
+	m_bbTCO->setName( m_bbTCO->getTrack()->name() );
 }
 
 
 
 
-void bbTCO::resetName( void )
+void bbTCOView::changeName( void )
 {
-	if( dynamic_cast<bbTrack *>( getTrack() ) != NULL )
-	{
-		m_name = dynamic_cast<bbTrack *>( getTrack() )->
-							trackLabel()->text();
-	}
-}
-
-
-
-
-void bbTCO::changeName( void )
-{
-	renameDialog rename_dlg( m_name );
+	QString s = m_bbTCO->name();
+	renameDialog rename_dlg( s );
 	rename_dlg.exec();
+	m_bbTCO->setName( s );
 }
 
 
 
 
-void bbTCO::changeColor( void )
+void bbTCOView::changeColor( void )
 {
-	QColor _new_color = QColorDialog::getColor( m_color );
+	QColor _new_color = QColorDialog::getColor( m_bbTCO->m_color );
 	if( !_new_color.isValid() )
 	{
 		return;
 	}
 	if( isSelected() )
 	{
-		vvector<selectableObject *> selected =
+		QVector<selectableObject *> selected =
 				engine::getSongEditor()->selectedObjects();
-		for( vvector<selectableObject *>::iterator it =
+		for( QVector<selectableObject *>::iterator it =
 							selected.begin();
 						it != selected.end(); ++it )
 		{
-			bbTCO * bb_tco = dynamic_cast<bbTCO *>( *it );
-			if( bb_tco )
+			bbTCOView * bb_tcov = dynamic_cast<bbTCOView *>( *it );
+			if( bb_tcov )
 			{
-				bb_tco->setColor( _new_color );
+				bb_tcov->setColor( _new_color );
 			}
 		}
 	}
@@ -319,12 +286,12 @@ void bbTCO::changeColor( void )
 
 
 
-void bbTCO::setColor( QColor _new_color )
+void bbTCOView::setColor( QColor _new_color )
 {
-	if( _new_color != m_color )
+	if( _new_color.rgb() != m_bbTCO->m_color )
 	{
-		m_color = _new_color;
-		engine::getSongEditor()->setModified();
+		m_bbTCO->m_color = _new_color.rgb();
+		engine::getSong()->setModified();
 		update();
 	}
 }
@@ -333,35 +300,20 @@ void bbTCO::setColor( QColor _new_color )
 
 
 
-bbTrack::bbTrack( trackContainer * _tc ) :
-	track( _tc )
-{
-	getTrackWidget()->setFixedHeight( 32 );
-	// drag'n'drop with bb-tracks only causes troubles (and makes no sense
-	// too), so disable it
-	getTrackWidget()->setAcceptDrops( FALSE );
 
-	csize bbNum = s_infoMap.size();
+
+bbTrack::bbTrack( trackContainer * _tc ) :
+	track( BBTrack, _tc )
+{
+	int bbNum = s_infoMap.size();
 	s_infoMap[this] = bbNum;
 
-	m_trackLabel = new nameLabel( tr( "Beat/Baseline %1" ).arg( bbNum ),
-						getTrackSettingsWidget() );
-	m_trackLabel->setPixmap( embed::getIconPixmap( "bb_track" ) );
-	m_trackLabel->setGeometry( 1, 1, DEFAULT_SETTINGS_WIDGET_WIDTH - 2,
-									29 );
-	m_trackLabel->show();
-	connect( m_trackLabel, SIGNAL( clicked() ),
-			this, SLOT( clickedTrackLabel() ) );
-	connect( m_trackLabel, SIGNAL( nameChanged() ),
-			engine::getBBEditor(), SLOT( updateComboBox() ) );
-	connect( m_trackLabel, SIGNAL( pixmapChanged() ),
-			engine::getBBEditor(), SLOT( updateComboBox() ) );
+	setName( tr( "Beat/Bassline %1" ).arg( bbNum ) );
+	engine::getBBTrackContainer()->setCurrentBB( bbNum );
+	engine::getBBTrackContainer()->updateComboBox();
 
-
-	engine::getBBEditor()->setCurrentBB( bbNum );
-	engine::getBBEditor()->updateComboBox();
-
-	_tc->updateAfterTrackAdd();
+	connect( this, SIGNAL( nameChanged() ),
+		engine::getBBTrackContainer(), SLOT( updateComboBox() ) );
 }
 
 
@@ -371,76 +323,68 @@ bbTrack::~bbTrack()
 {
 	engine::getMixer()->removePlayHandles( this );
 
-	csize bb = s_infoMap[this];
-	engine::getBBEditor()->removeBB( bb );
+	const int bb = s_infoMap[this];
+	engine::getBBTrackContainer()->removeBB( bb );
 	for( infoMap::iterator it = s_infoMap.begin(); it != s_infoMap.end();
 									++it )
 	{
-#ifdef QT4
 		if( it.value() > bb )
 		{
 			--it.value();
 		}
-#else
-		if( it.data() > bb )
-		{
-			--it.data();
-		}
-#endif
 	}
 	s_infoMap.remove( this );
-	engine::getBBEditor()->updateComboBox();
+
+	// remove us from TC so bbTrackContainer::numOfBBs() returns a smaller
+	// value and thus combobox-updating in bbTrackContainer works well
+	getTrackContainer()->removeTrack( this );
+	engine::getBBTrackContainer()->updateComboBox();
 }
 
 
 
 
-track::trackTypes bbTrack::type( void ) const
-{
-	return( BB_TRACK );
-}
-
-
-
 // play _frames frames of given TCO within starting with _start
-bool FASTCALL bbTrack::play( const midiTime & _start,
-						const fpp_t _frames,
-						const f_cnt_t _offset,
-							Sint16 _tco_num )
+bool bbTrack::play( const midiTime & _start, const fpp_t _frames,
+				const f_cnt_t _offset, Sint16 _tco_num )
 {
-	sendMidiTime( _start );
+	if( isMuted() )
+	{
+		return( FALSE );
+	}
 
 	if( _tco_num >= 0 )
 	{
-		return( engine::getBBEditor()->play( _start, _frames,
+		return( engine::getBBTrackContainer()->play( _start, _frames,
 							_offset,
 							s_infoMap[this] ) );
 	}
 
-	vlist<trackContentObject *> tcos;
-	getTCOsInRange( tcos, _start, _start + static_cast<Sint32>( _frames /
-						engine::framesPerTact64th() ) );
+	tcoVector tcos;
+	getTCOsInRange( tcos, _start, _start + static_cast<int>( _frames /
+						engine::framesPerTick() ) );
 
-	if ( tcos.size() == 0 )
+	if( tcos.size() == 0 )
 	{
 		return( FALSE );
 	}
 
 	midiTime lastPosition;
 	midiTime lastLen;
-	for( vlist<trackContentObject *>::iterator it = tcos.begin();
-							it != tcos.end(); ++it )
+	for( tcoVector::iterator it = tcos.begin(); it != tcos.end(); ++it )
 	{
-		if( !( *it )->muted() &&
+		if( !( *it )->isMuted() &&
 				( *it )->startPosition() >= lastPosition )
 		{
 			lastPosition = ( *it )->startPosition();
 			lastLen = ( *it )->length();
 		}
 	}
+
 	if( _start - lastPosition < lastLen )
 	{
-		return( engine::getBBEditor()->play( _start - lastPosition,
+		return( engine::getBBTrackContainer()->play( _start -
+								lastPosition,
 							_frames,
 							_offset,
 							s_infoMap[this] ) );
@@ -451,12 +395,20 @@ bool FASTCALL bbTrack::play( const midiTime & _start,
 
 
 
+trackView * bbTrack::createView( trackContainerView * _tcv )
+{
+	return( new bbTrackView( this, _tcv ) );
+}
+
+
+
+
 trackContentObject * bbTrack::createTCO( const midiTime & _pos )
 {
 	// if we're creating a new bbTCO, we colorize it according to the
 	// previous bbTCO, so we have to get all TCOs from 0 to _pos and
 	// pickup the last and take the color if it
-	vlist<trackContentObject *> tcos;
+	tcoVector tcos;
 	getTCOsInRange( tcos, 0, _pos );
 	if( tcos.size() > 0 && dynamic_cast<bbTCO *>( tcos.back() ) != NULL )
 	{
@@ -475,30 +427,19 @@ trackContentObject * bbTrack::createTCO( const midiTime & _pos )
 void bbTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 							QDomElement & _this )
 {
-	_this.setAttribute( "name", m_trackLabel->text() );
-	_this.setAttribute( "icon", m_trackLabel->pixmapFile() );
+//	_this.setAttribute( "icon", m_trackLabel->pixmapFile() );
 /*	_this.setAttribute( "current", s_infoMap[this] ==
 					engine::getBBEditor()->currentBB() );*/
 	if( s_infoMap[this] == 0 &&
 			_this.parentNode().parentNode().nodeName() != "clone" &&
 			_this.parentNode().nodeName() != "journaldata" )
 	{
-		( (journallingObject *)( engine::getBBEditor() ) )->saveState(
-								_doc, _this );
+		( (journallingObject *)( engine::getBBTrackContainer() ) )->
+						saveState( _doc, _this );
 	}
-
-	int track_num = 0;
-	trackVector tracks = engine::getBBEditor()->tracks();
-	for( trackVector::iterator it = tracks.begin(); it != tracks.end();
-							++it, ++track_num )
+	if( _this.parentNode().parentNode().nodeName() == "clone" )
 	{
-		if( automationDisabled( *it ) )
-		{
-			QDomElement disabled = _doc.createElement(
-							"automation-disabled" );
-			disabled.setAttribute( "track", track_num );
-			_this.appendChild( disabled );
-		}
+		_this.setAttribute( "clonebbt", s_infoMap[this] );
 	}
 }
 
@@ -507,18 +448,34 @@ void bbTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 
 void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 {
-	m_trackLabel->setText( _this.attribute( "name" ) );
-	if( _this.attribute( "icon" ) != "" )
+/*	if( _this.attribute( "icon" ) != "" )
 	{
 		m_trackLabel->setPixmapFile( _this.attribute( "icon" ) );
-	}
-	engine::getBBEditor()->updateComboBox();
+	}*/
 
-	QDomNode node = _this.namedItem( trackContainer::classNodeName() );
-	if( node.isElement() )
+	if( _this.hasAttribute( "clonebbt" ) )
 	{
-		( (journallingObject *)engine::getBBEditor() )->restoreState(
-							node.toElement() );
+		const int src = _this.attribute( "clonebb" ).toInt();
+		const int dst = s_infoMap[this];
+		engine::getBBTrackContainer()->createTCOsForBB( dst );
+		trackContainer::trackList tl =
+					engine::getBBTrackContainer()->tracks();
+		for( trackContainer::trackList::iterator it = tl.begin();
+							it != tl.end(); ++it )
+		{
+			( *it )->getTCO( src )->copy();
+			( *it )->getTCO( dst )->paste();
+		}
+	}
+	else
+	{
+		QDomNode node = _this.namedItem(
+					trackContainer::classNodeName() );
+		if( node.isElement() )
+		{
+			( (journallingObject *)engine::getBBTrackContainer() )->
+					restoreState( node.toElement() );
+		}
 	}
 /*	doesn't work yet because bbTrack-ctor also sets current bb so if
 	bb-tracks are created after this function is called, this doesn't
@@ -527,35 +484,18 @@ void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 	{
 		engine::getBBEditor()->setCurrentBB( s_infoMap[this] );
 	}*/
-
-	trackVector tracks = engine::getBBEditor()->tracks();
-	node = _this.firstChild();
-	while( !node.isNull() )
-	{
-		if( node.isElement()
-				&& node.nodeName() == "automation-disabled" )
-		{
-			disableAutomation( tracks[node.toElement().attribute(
-							"track" ).toInt()] );
-		}
-		node = node.nextSibling();
-	}
 }
 
 
 
 
 // return pointer to bbTrack specified by _bb_num
-bbTrack * bbTrack::findBBTrack( csize _bb_num )
+bbTrack * bbTrack::findBBTrack( int _bb_num )
 {
 	for( infoMap::iterator it = s_infoMap.begin(); it != s_infoMap.end();
 									++it )
 	{
-#ifdef QT4
 		if( it.value() == _bb_num )
-#else
-		if( it.data() == _bb_num )
-#endif
 		{
 			return( it.key() );
 		}
@@ -566,7 +506,7 @@ bbTrack * bbTrack::findBBTrack( csize _bb_num )
 
 
 
-csize bbTrack::numOfBBTrack( track * _track )
+int bbTrack::numOfBBTrack( track * _track )
 {
 	if( dynamic_cast<bbTrack *>( _track ) != NULL )
 	{
@@ -585,24 +525,75 @@ void bbTrack::swapBBTracks( track * _track1, track * _track2 )
 	if( t1 != NULL && t2 != NULL )
 	{
 		qSwap( s_infoMap[t1], s_infoMap[t2] );
-		engine::getBBEditor()->swapBB( s_infoMap[t1], s_infoMap[t2] );
-		engine::getBBEditor()->setCurrentBB( s_infoMap[t1] );
+		engine::getBBTrackContainer()->swapBB( s_infoMap[t1],
+								s_infoMap[t2] );
+		engine::getBBTrackContainer()->setCurrentBB( s_infoMap[t1] );
 	}
 }
 
 
 
 
-void bbTrack::clickedTrackLabel( void )
+
+
+
+
+
+bbTrackView::bbTrackView( bbTrack * _bbt, trackContainerView * _tcv ) :
+	trackView( _bbt, _tcv ),
+	m_bbTrack( _bbt )
 {
-	engine::getBBEditor()->setCurrentBB( s_infoMap[this] );
-	engine::getBBEditor()->show();
+	setFixedHeight( 32 );
+	// drag'n'drop with bb-tracks only causes troubles (and makes no sense
+	// too), so disable it
+	setAcceptDrops( FALSE );
+
+	m_trackLabel = new trackLabelButton( this, getTrackSettingsWidget() );
+	m_trackLabel->setIcon( embed::getIconPixmap( "bb_track" ) );
+	m_trackLabel->move( 3, 1 );
+	m_trackLabel->show();
+	connect( m_trackLabel, SIGNAL( clicked( bool ) ),
+			this, SLOT( clickedTrackLabel() ) );
+	setModel( _bbt );
 }
 
 
 
 
-#include "bb_track.moc"
+bbTrackView::~bbTrackView()
+{
+	engine::getBBEditor()->removeBBView( bbTrack::s_infoMap[m_bbTrack] );
+}
+
+
+
+
+bool bbTrackView::close( void )
+{
+	engine::getBBEditor()->removeBBView( bbTrack::s_infoMap[m_bbTrack] );
+	return( trackView::close() );
+}
+
+
+
+
+void bbTrackView::clickedTrackLabel( void )
+{
+	engine::getBBTrackContainer()->setCurrentBB(
+					bbTrack::numOfBBTrack( m_bbTrack ) );
+	engine::getBBEditor()->show();
+/*	foreach( bbTrackView * tv,
+			getTrackContainerView()->findChildren<bbTrackView *>() )
+	{
+		tv->m_trackLabel->update();
+	}*/
+	
+}
+
+
+
+
+#include "moc_bb_track.cxx"
 
 
 #endif
